# Copyright(c) 1999, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: Contextlite.tcl,v 1.1 2004/06/18 20:30:20 tsmith Exp $

#-------------------------------------------------------------------------------#
# RestoreContext -- restores all context file data from disk                    #
#-------------------------------------------------------------------------------#
proc RestoreContext { } \
  {
  set ContextName [GetLocalDataPath "CONTEXT.DAT"]
  if [ catch { open $ContextName r } fileID ] { set fileID -1 }

  CTRestoreLogon		$fileID ;# Restores the Logon Data
  CTRestoreCursor 		$fileID	;# Restores the Cursor Data
  CTRestoreLoops 		$fileID ;# Restores loops defined in the GUI
  CTRestoreMacros 		$fileID ;# Restores the user defined macro's
#ml  CTRestoreDwellRate 	$fileID	;# Restores the dwell rate
#ml  CTRestoreGUICmdEcho 	$fileID	;# Restores the GUI command echo flag
#ml  CTRestoreSchedFile  	$fileID	;# Restores the name of the scheduler file
#ml  CTRestoreToolbars 	$fileID	;# Restores settings for showing/hiding toolbars

  catch { close $fileID }
  return  
  }

#-------------------------------------------------------------------------------#
# SaveContext -- saves all context file data to disk                            #
#-------------------------------------------------------------------------------#
proc SaveContext { } \
{
  #---Get full path to context file
  set ContextName [GetLocalDataPath "CONTEXT.DAT"]

  #---Move the existing context file to "CONTEXT.DAT.BAK"
  if { [file exists $ContextName] } \
  {
      file rename -force $ContextName $ContextName.BAK
  }

  #--- Create a new context file
  if [ catch { open $ContextName w } fileID ] \
  {
      # error message if you want one
      return
  }
  
  CTSaveLogon 		$fileID ;# Saves the Logon data
  CTSaveCursor 		$fileID ;# Saves the Cursor data  
  CTSaveLoops  		$fileID ;# Saves loops defined in the GUI
  CTSaveMacros 		$fileID ;# Saves Macro's defined in the GUI
#ml  CTSaveDwellRate 	$fileID ;# Saves the dwell rate
#ml  CTSaveGUICmdEcho 	$fileID ;# Saves the command echo flag
#ml  CTSaveSchedFile	$fileID ;# Saves the scheduler file name
#ml  CTSaveToolbars	$fileID ;# Saves the toolbar settings

  catch { close $fileID }
}


proc CTSaveLogon { fileID } \
  {
  global Initials		; # Logon identifying initials
  global ProjectNumber		; # Project number to charge too
  
  puts $fileID "LOGON VARIABLES"
  puts $fileID "   Initials      $Initials"
  puts $fileID "   ProjectNumber $ProjectNumber"
  } 
proc CTRestoreLogon { fileID } \
  {
  global Initials		; # Logon identifying initials
  global ProjectNumber		; # Project number to charge too

  # No file so reset
  if { $fileID==-1} \
    {
    set Initials ""
    set ProjectNumber ""
    return
    }
    
  #--- burn Header
    gets $fileID line  
  #--- INITIALS
    gets $fileID line
    set Initials [lindex $line 1]
  #--- PROJECTNUMBER  
    gets $fileID line
    set ProjectNumber [lindex $line 1]
  }   
  

proc CTSaveCursor { fileID } \
  {
  global CursorType
  global CursorWidth
  global CursorHeight
  global CursorColor
  
  puts $fileID "CURSOR VARIABLES"
  puts $fileID "   CursorType    $CursorType"
  puts $fileID "   CursorWidth   $CursorWidth"
  puts $fileID "   CursorHeight  $CursorHeight"
  puts $fileID "   CursorColor   $CursorColor"
  }  
proc CTRestoreCursor { fileID } \
  {
  global CursorType		;# The type of cursor (4 types)
  global CursorWidth		;# Cursor Width
  global CursorHeight		;# Cursor Height
  global CursorColor		;# RGB string of cursor color #RRGGBB format
  
  # No file so reset
  if { $fileID==-1} \
    {
    set CursorType 2
    set CursorWidth 31
    set CursorHeight 31
    set CursorColor "#ff0000"
    McSetCursor
    return
    }
  #--- Burn the header
    gets $fileID line
  
  #--- TYPE
    gets $fileID line
    set CursorType [lindex $line 1]
  #--- WIDTH  
    gets $fileID line
    set CursorWidth [lindex $line 1]
  #--- HEIGHT  
    gets $fileID line
    set CursorHeight [lindex $line 1]
  #--- COLOR  
    gets $fileID line
    set CursorColor [lindex $line 1]
  #--- Call routine to set the variables 
    McSetCursor  
  }  

proc CTSaveLoops { fileID } \
  {
  global LMLoopCount    ;# -- Number of defined loops
  global LMLoops        ;# -- Contains the Loop Names and data in an array as follows
                         # -- "#,N" -- Holds the NAME of the loop
		         # -- "#,F" -- Holds the number of FRAMES in the loop
                         # -- "#,O" -- List of the order of the frames in the loop 

  puts $fileID "LOOP VARIABLES"
  puts $fileID "   LMLoopCount  $LMLoopCount"
  for { set i 0 } { $i < $LMLoopCount } { incr i } \
    {
    # --- reset the -# loops to positive so that they are valid once again
    set FrameNum $LMLoops($i,F)
    if { $FrameNum < 0 } { set FrameNum [expr -$FrameNum] }
    puts $fileID "   NAME:   $LMLoops($i,N)"
    puts $fileID "   FRAMES: $FrameNum"
    puts $fileID "   ORDER:  $LMLoops($i,O)"
    }
  }    
proc CTRestoreLoops { fileID } \
  {
  global LMLoopCount    ;# -- Number of defined loops
  global LMLoops        ;# -- Contains the Loop Names and data in an array as follows
                         # -- "#,N" -- Holds the NAME of the loop
		         # -- "#,F" -- Holds the number of FRAMES in the loop
                         # -- "#,O" -- List of the order of the frames in the loop 
  global LMFrameCount   ;# -- supported number of frames			 
  
  # No file so reset
  if { $fileID==-1} \
    {
    set LMLoopCount 0
    LMSetFrameAndLoop  1 -2  ; # Frame 1 -- loop -2 (empty frames)
    return
    }

  
  #--- Burn the header
   gets $fileID line

  #--- LOOP COUNT
    gets $fileID line
    set LMLoopCount [lindex $line 1]
   
  #--- Restore the LOOP ARRAY
    if { $LMLoopCount } \
      {
      for { set i 0 } { $i < $LMLoopCount } { incr i } \
        {
        #--- NAME
          gets $fileID line
	  set LMLoops($i,N) [lrange $line 1 end]
        #--- FRAMES (Number of)
          gets $fileID line
          set LMLoops($i,F) [lindex $line 1]
        #--- ORDER of FRAMEs
          gets $fileID line
          set LMLoops($i,O) [lrange $line 1 end]
        }
      # --- scan through the loops and toss any with a frame out of range
      for { set i 0 } { $i < $LMLoopCount } { incr i } \
        {
	for {set j 0} { $j < $LMLoops($i,F) } { incr j } \
	  {
	  if { [lindex $LMLoops($i,O) $j] >= $LMFrameCount } \
	    {
	    # Invalidate the loop but don't remove -- that way we can save it back anyway
	    set LMLoops($i,F) [expr -$LMLoops($i,F)]
	    set j 10000    ;# break out of the J loop
	    }
	  }
	}
      }	
  # -- find the first valid loop and restore it
  for { set i 0 } { $i < $LMLoopCount } { incr i } \
    {
    if { $LMLoops($i,F)>0 } \
      {
      LMSetFrameAndLoop  1 $i
      return
      }
    }      
  # -- made it all the way through, set to the empty frames
    LMSetFrameAndLoop  1 -2  ; # Frame 1 -- loop -2 (empty frames)
  }  
  
  
proc CTSaveMacros { fileID } \
  {
  global MCMacros        ;# Macros List
  global MCMacroNum      ;# Number of defined macros
  
  puts $fileID "MACRO NAMES AND STRINGS"
  puts $fileID "   NUMBER: $MCMacroNum"   ;# Number of Macros
  for { set i 0 } { $i<$MCMacroNum } { incr i } \
    {
    puts $fileID "   NAME:   $MCMacros($i,N)"   
    puts $fileID "   STRING: $MCMacros($i,C)"   
    }  
  }  
proc CTRestoreMacros { fileID } \
  {
  global MCMacros        ;# Macros List
  global MCMacroNum      ;# Number of defined macros
  global MCCount         ;# Number of macro buttons to show
  
  set MCCount 4

  # No file so reset
  if { $fileID==-1} \
    {
    set MCMacroNum 0
    return
    }
  #--- Burn the header
    gets $fileID line
  
  #--- Number
    gets $fileID line
    set MCMacroNum [lindex $line 1]
    
  #--- Now restore them all  
    for { set i 0 } { $i < $MCMacroNum } { incr i } \
      {
      gets $fileID line
      set MCMacros($i,N) [string trim [string range $line 10 end]]
      gets $fileID line
      set MCMacros($i,C) [string trim [string range $line 10 end]]
      }
  #--- make sure that NUMBER is defined
   if { ![string length $MCMacroNum] } { set MCMacroNum 0 }
  }

proc CTSaveDwellRate { fileID } \
{

  global TB_loop_dwell_scale
  global dwell_pause
  
  puts $fileID "DWELL RATE VARIABLES"
  puts $fileID "   Dwell Rate  [$TB_loop_dwell_scale get]"
  puts $fileID "   Dwell Pause $dwell_pause"
}

proc CTRestoreDwellRate { fileID } \
{  
  global initial_dwell_rate
  global initial_pause_status

  #--- No file so reset
  if { $fileID == -1} \
  {
	set initial_dwell_rate 6
	set initial_pause_status "OFF"
	return
  }

  #--- Burn the header
  gets $fileID line

  #--- Get the dwell rate
  gets $fileID line
  set initial_dwell_rate [lindex $line 2]

  #--- Get the dwell rate pause status
  gets $fileID line
  set initial_pause_status [lindex $line 2]
}

proc CTSaveGUICmdEcho { fileID } \
{
  puts $fileID "GUI COMMAND ECHOING"
  puts $fileID "   Echo [UCpeek [ucLocation gui_cmd_echo]]"
}

proc CTRestoreGUICmdEcho { fileID } \
{  
  global env

  #--- No file so reset
  if { $fileID == -1} \
  {
	UCpoke [ucLocation gui_cmd_echo] 0
	return
  }

  #--- Get the command history flag value
  gets $fileID line
  gets $fileID line
  catch { UCpoke [ucLocation gui_cmd_echo] [lindex $line 1] }

  #--- Initialize the GUI command echo checkbutton
  set ::echo_gui_commands [UCpeek [ucLocation gui_cmd_echo]]
}

proc CTSaveSchedFile { fileID } \
{
  global DefaultSchedulerFile

  puts $fileID "DEFAULT SCHEDULER FILE"

  set file_name "SKEDFILE"

  catch \
  {
	if { [string length [string trim $DefaultSchedulerFile]] == 0 } \
 	{
	   set file_name "SKEDFILE"
	} \
	else \
	{
	   set file_name $DefaultSchedulerFile
	}
  }

  puts $fileID "   File name  $file_name"
}

proc CTRestoreSchedFile { fileID } \
{
  global DefaultSchedulerFile

  #--- No file so reset
  if { $fileID == -1} \
  {
	set DefaultSchedulerFile "SKEDFILE"
	return
  }

  #--- Get the command history flag value
  gets $fileID line
  gets $fileID line

  if { [string match "*File name*" $line] } \
  {
	set DefaultSchedulerFile [lindex $line 2]
  } \
  else \
  {
	set DefaultSchedulerFile "SKEDFILE"
  }
}


proc CTSaveToolbars { fileID } \
{
   global TBUserButtonFlag
   global TBImageControlFlag
   global TBFrameControlFlag
   global TBStatusBarFlag


   puts $fileID "TOOLBAR STATUS"

   puts $fileID "   UserDefinedToolbar  $TBUserButtonFlag"
   puts $fileID "   ImageControlToolbar $TBImageControlFlag"
   puts $fileID "   FrameControlToolbar $TBFrameControlFlag"
   puts $fileID "   StatusBar           $TBStatusBarFlag"
}


proc CTRestoreToolbars { fileID } \
{
   global TBUserButtonFlag
   global TBImageControlFlag
   global TBFrameControlFlag
   global TBStatusBarFlag
 
  #--- Set defaults
  set TBUserButtonFlag   BOTTOM
  set TBImageControlFlag BOTTOM
  set TBFrameControlFlag BOTTOM
  set TBStatusBarFlag    BOTTOM

  #--- No file so reset
  if { $fileID == -1} { return }

  gets $fileID line ;# burn header

  gets $fileID line
  if { [string match "*UserDefinedToolbar*" $line] } \
  {
  	set TBUserButtonFlag [lindex $line 1]
  }

  gets $fileID line
  if { [string match "*ImageControlToolbar*" $line] } \
  {
	set TBImageControlFlag [lindex $line 1]
  }

  gets $fileID line
  if { [string match "*FrameControlToolbar*" $line] } \
  {
	set TBFrameControlFlag [lindex $line 1]
  }

  gets $fileID line
  if { [string match "*StatusBar*" $line] } \
  {
	set TBStatusBarFlag [lindex $line 1]
  }
}

########### End of File #######################################
