# Copyright(c) 1998, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: MyGridSelect.tcl,v 1.27 2007/04/24 19:50:12 davep Exp $

#-----------------------------------------------------------------------------------
#
#
#===================================================================================#
#-----------------------------------------------------------------------------------#
# proc GridSelect { } -- Upper level code to organize the GRIDDISP command
#-----------------------------------------------------------------------------------#
proc GridSelect { } \
   {
   setoptions
   global listboxfont

   #---- Make sure that the arrays are defined
   global MGSActiveServer MGSActiveEntry
   global MGSEntries MGSEntriesList MGSFilteredList
   global MGSDate enMGSDataSet MGSjDate
   global MGSLBox MGSMenues MGSSelectionChange
   global MGSProjection MGSParameter MGSLevel
   global MGSContourColor MGSDash MGSMapColor MGSSource
   global MGSLabelInterval MGSLabelHeight
   global MGSSLat MGSSLon
   global MGSMapOptions  ; # remove later

   if { [winfo exists .tlGridSelect] } { wm deiconify .tlGridSelect; focus .tlGridSelect; raise .tlGridSelect; return }

   #--- Initialize the list data correctly (use last active Server if applicable else first in list)
     if { ! [info exists MGSActiveServer] } { set MGSActiveServer 0 }
     if { ! [info exists MGSActiveEntry] }  { set MGSActiveEntry 0 }
     MGSBuildList

   #====================== Initialize all of the data ===========================
     set MGSSource ""
     set MGSProjection ""
     set MGSLevel ""
     set MGSDate ""
     set MGSjDate -1
     set MGSSelectionChange 0  ;# List box hasn't been clicked
     set MGSContourColor 2
     set MGSMapColor 1
     set MGSMapOptions "NAV=C"
     set MGSSLat ""
     set MGSSLon ""

   toplevel .tlGridSelect -borderwidth 3 -relief raised
   wm title .tlGridSelect "Grid Select"
   wm geometry .tlGridSelect +5+5
 
   frame .tlGridSelect.fmBody
     Notebook:create .tlGridSelect.nbGridDisp -pages { Data Options } -pad 10\
        -width 500 -height 500
     pack .tlGridSelect.nbGridDisp -fill both -expand true
     #---- DATA section ---------------------------------------
     set data [Notebook:frame .tlGridSelect.nbGridDisp Data]
       #---- Menues frame
       frame $data.frameMenu
         LMCreateLoopButton $data.frameMenu "Select Frame" YES
         #--- Dataset menu
         frame $data.frameMenu.frameDS
	 pack $data.frameMenu.frameDS -side top -anchor w -fill x 
         #-- Source frame
         frame $data.frameMenu.frameSource
	 pack $data.frameMenu.frameSource -side top -anchor w -fill x
         #-- Projection frame
         frame $data.frameMenu.frameProjection
         pack $data.frameMenu.frameProjection -side top -anchor w -fill x
         #-- Parameter frame
         frame $data.frameMenu.frameParameter
         pack $data.frameMenu.frameParameter -side top -anchor w -fill x
         #-- Level frame
         frame $data.frameMenu.frameLevel
         pack $data.frameMenu.frameLevel -side top -anchor w -fill x
	 #--- Date Range
         frame $data.frameMenu.frameDate
	 pack $data.frameMenu.frameDate -side top -anchor w -fill x 
       #--Listbox frame      	      
         frame $data.frameMenu.frameList
	 pack $data.frameMenu.frameList -side top -fill both -expand true
       pack $data.frameMenu -side top -pady 10 -padx 10 -fill both -expand true
       set MGSMenues $data.frameMenu	 
     #---- OPTIONS section ---------------------------------------
     set options [Notebook:frame .tlGridSelect.nbGridDisp Options]
       #-- Main frame
       frame $options.frameMenu
       #-- Map Options
         frame $options.frameMenu.frameMapOptions -relief raised -borderwidth 3
           MGSMapOptionsMenu $options.frameMenu.frameMapOptions 
	 pack $options.frameMenu.frameMapOptions  -side top -fill x
       #--Contour units
         frame $options.frameMenu.frameContourUnits -relief raised -borderwidth 3
            MGSContourUnitsMenu $options.frameMenu.frameContourUnits
	 pack $options.frameMenu.frameContourUnits -side top -fill x
       #--Contour Intervals
         frame $options.frameMenu.frameContourIntervals -relief raised -borderwidth 3
	    MGSIntervalMenu $options.frameMenu.frameContourIntervals
	 pack $options.frameMenu.frameContourIntervals -side top -fill x
       #--Contour Intervals Color
         frame $options.frameMenu.frameContourColor -relief raised -borderwidth 3
            button $options.frameMenu.frameContourColor.buttonSelectColor -text "Contour Color" -borderwidth 2 -width 17 \
      		-command {set MGSContourColor [ColorChooser $MGSContourColor]
		          set options [Notebook:frame .tlGridSelect.nbGridDisp Options]
                          $options.frameMenu.frameContourColor.labelColor config -bg [ColorLevelToRGB $MGSContourColor]}
            label $options.frameMenu.frameContourColor.labelColor -highlightthickness 0 -relief groove -width 2 -bg [ColorLevelToRGB $MGSContourColor]
            pack $options.frameMenu.frameContourColor.buttonSelectColor -side left 
            pack $options.frameMenu.frameContourColor.labelColor -side right -fill x -expand true -padx 2
         pack $options.frameMenu.frameContourColor -side top -fill x 
       #--Dash Option
         frame $options.frameMenu.frameDash -relief raised -borderwidth 3
           MGSDashMenu $options.frameMenu.frameDash
	 pack  $options.frameMenu.frameDash -side top -fill x
       #--Label Interval/Height
         frame $options.frameMenu.frameLInterval -relief raised -borderwidth 3
         frame $options.frameMenu.frameLHeight -relief raised -borderwidth 3
           MGSLabelMenues $options.frameMenu.frameLInterval $options.frameMenu.frameLHeight
         pack $options.frameMenu.frameLInterval -side top -fill x
         pack $options.frameMenu.frameLHeight -side top -fill x
       #--Map Color
         frame $options.frameMenu.frameMapColor -relief raised -borderwidth 3
            button $options.frameMenu.frameMapColor.buttonSelectColor -text "Map Color" -borderwidth 2 -width 17 \
      		-command {set MGSMapColor [ColorChooser $MGSMapColor]
		          set options [Notebook:frame .tlGridSelect.nbGridDisp Options]
                          $options.frameMenu.frameMapColor.labelColor config -bg [ColorLevelToRGB $MGSMapColor]}
            label $options.frameMenu.frameMapColor.labelColor -highlightthickness 0 -relief groove -width 2 -bg [ColorLevelToRGB $MGSMapColor]
            pack $options.frameMenu.frameMapColor.buttonSelectColor -side left 
            pack $options.frameMenu.frameMapColor.labelColor -side right -fill x -expand true -padx 2
         pack $options.frameMenu.frameMapColor -side top -fill x 
       #--Projection
         frame $options.frameMenu.frameProjection -relief raised -borderwidth 3
           MGSProjectionMenu $options.frameMenu.frameProjection
	 pack  $options.frameMenu.frameProjection -side top -fill x
       #--SLat and SLon entry fields
         frame $options.frameMenu.frameSLat -relief raised -borderwidth 3
	   label $options.frameMenu.frameSLat.lbLat -text "Standard Lat:" -width 19 -relief flat -borderwidth 4
	   entry $options.frameMenu.frameSLat.enLat \
		-textvar MGSSLat -width 26 -state disabled
	   pack $options.frameMenu.frameSLat.lbLat -side left
	   pack $options.frameMenu.frameSLat.enLat -side left -fill x -expand true
	 pack $options.frameMenu.frameSLat -side top -fill x
         frame $options.frameMenu.frameSLon -relief raised -borderwidth 3
	   label $options.frameMenu.frameSLon.lbLon -text "Standard Lon:" -width 19 -relief flat -borderwidth 4
	   entry $options.frameMenu.frameSLon.enLon \
		-textvar MGSSLon -width 26 -state disabled
	   pack $options.frameMenu.frameSLon.lbLon -side left
	   pack $options.frameMenu.frameSLon.enLon -side left -fill x -expand true
	 pack $options.frameMenu.frameSLon -side top -fill x
       pack $options.frameMenu -side top -pady 10 -padx 10 -fill both -expand true
   pack .tlGridSelect.fmBody  

   #---Controls frame (load/dismiss/load&dismiss)
   frame .tlGridSelect.fmBottom 
     button .tlGridSelect.fmBottom.btLoad -text "Load Grid(s)" -command MGSLoadGrids
     button .tlGridSelect.fmBottom.btLoadDismiss -text "Load and Dismiss" -command  { MGSLoadGrids; destroy .tlGridSelect }
     button .tlGridSelect.fmBottom.btDismiss -text "Dismiss" -command {destroy .tlGridSelect }
     pack .tlGridSelect.fmBottom.btLoad -side left -padx 10 -pady 5
     pack .tlGridSelect.fmBottom.btLoadDismiss -side left -padx 10 -pady 5
     pack .tlGridSelect.fmBottom.btDismiss -side right -padx 10 -pady 5
   pack .tlGridSelect.fmBottom -padx 20 -pady 5


   #------------ Fill the menues here ----------------------------
     set f $data.frameMenu.frameDS
     set enMGSDataSet ""
     
     menubutton $f.mb -text "Dataset" -menu $f.mb.menu \
                      -relief raised -borderwidth 2 -indicatoron 1 -width 12
     pack  $f.mb -side left
     label $f.entryDS -textvar enMGSDataSet -width 26 \
	-relief sunken -borderwidth 2 -anchor w
     pack  $f.entryDS -side left -fill x -expand true
     set m [menu $f.mb.menu -tearoff 0]
     set ServerIndex 0
     while { [info exists MGSEntries($ServerIndex)] } \
     {
       $m add cascade -label $MGSEntries($ServerIndex) -menu $m.sub$ServerIndex 
       set EntryIndex 0
       set m2 [menu $m.sub$ServerIndex -tearoff 0]

     set counter 1
     set column_break_flag "NO"

       while { [info exists MGSEntries($ServerIndex,$EntryIndex)] }\
       {
           $m2 add command \
		-label [lindex $MGSEntries($ServerIndex,$EntryIndex) 0] \
		-command "MGSDoList $ServerIndex $EntryIndex" \
		-columnbreak $column_break_flag

	   # Set for columns of 20 so we don't go off of the bottom of the screen
	   if { [expr {$counter % 20}] == 0 } \
	   {
		set column_break_flag "YES"
	   } \
	   else \
 	   {
	   	set column_break_flag "NO"
	   }

	   incr counter
	   incr EntryIndex
       }

       incr ServerIndex   
     }   
   
    #--Source Menu
      set f $data.frameMenu
      menubutton $f.frameSource.mb -text "Source" -menu $f.frameSource.mb.menu \
                               -relief raised -borderwidth 2 -indicatoron 1 -width 12
      pack  $f.frameSource.mb -side left
      label $f.frameSource.entrySource \
	-textvar MGSSource -width 26  \
	-relief sunken -borderwidth 2 -anchor w
      pack  $f.frameSource.entrySource -side left -fill x -expand true
      pack $f.frameSource -side top -fill x 
      set m [menu $f.frameSource.mb.menu -tearoff 0]
    #--Projection Menu
      menubutton $f.frameProjection.mb -text "Projection" -menu $f.frameProjection.mb.menu \
                               -relief raised -borderwidth 2 -indicatoron 1 -width 12
      pack  $f.frameProjection.mb -side left
      label $f.frameProjection.entryProjection \
	-textvar MGSProjection -width 26  \
	-relief sunken -borderwidth 2 -anchor w
      pack  $f.frameProjection.entryProjection -side left -fill x -expand true
      pack  $f.frameProjection -side top -fill x 
      set m [menu $f.frameProjection.mb.menu -tearoff 0]
    #--Parameter Menu
      set MGSParameter ""
      menubutton $f.frameParameter.mb -text "Parameter" -menu $f.frameParameter.mb.menu \
                                 -relief raised -borderwidth 2 -indicatoron 1 -width 12
      pack  $f.frameParameter.mb -side left
      label $f.frameParameter.entryParam \
	-textvar MGSParameter -width 26  \
	-relief sunken -borderwidth 2 -anchor w
      pack  $f.frameParameter.entryParam -side left -fill x -expand true
      pack  $f.frameParameter -side top -fill x 
      set m [menu $f.frameParameter.mb.menu -tearoff 0]
    #--Level Menu
      menubutton $f.frameLevel.mb -text "Level" -menu $f.frameLevel.mb.menu \
                               -relief raised -borderwidth 2 -indicatoron 1 -width 12
      pack  $f.frameLevel.mb -side left
      label $f.frameLevel.entryLevel \
	-textvar MGSLevel -width 26  \
	-relief sunken -borderwidth 2 -anchor w
      pack  $f.frameLevel.entryLevel -side left -fill x -expand true
      pack  $f.frameLevel -side top -fill x 
      set m [menu $f.frameLevel.mb.menu -tearoff 0]
    #--Date Menu
      menubutton $f.frameDate.mb -text "Date" -menu $f.frameDate.mb.menu \
                               -relief raised -borderwidth 2 -indicatoron 1 -width 12
      pack  $f.frameDate.mb -side left
      label $f.frameDate.entryDate \
	-textvar MGSDate -width 26  \
	-relief sunken -borderwidth 2 -anchor w
      pack  $f.frameDate.entryDate -side left -fill x -expand true
      pack  $f.frameDate -side top -fill x 
      set m [menu $f.frameDate.mb.menu -tearoff 0]

    #-- The List box -- need MGSLBox to be defined for DataSet section
      label $data.frameMenu.frameList.spacer
      pack $data.frameMenu.frameList.spacer \
	-side top
      label $data.frameMenu.frameList.header1 \
	-text "Day                 Time         FHour FDay           FTime" \
	-anchor w 
      pack $data.frameMenu.frameList.header1 \
	-side top \
	-anchor w \
	-fill x

      set MGSLBox [MakeListBox $data.frameMenu.frameList "" ""]
      $MGSLBox config -selectmode extended -font $listboxfont

    }
#-----------------------------------------------------------------------------------#
# proc MGSSLatLon { enable } -- either enables the Lat/Lon boxes or disables them   #
#-----------------------------------------------------------------------------------#   
proc MGSSLatLon { enable } \
  {
  global back                ; # --- background color
  global activeback          ; # --- active background color

  set options [Notebook:frame .tlGridSelect.nbGridDisp Options]

  if { $enable==0 } \
    {
    # -- disable the controls
      $options.frameMenu.frameSLat.enLat config -state disabled -bg $back
      $options.frameMenu.frameSLon.enLon config -state disabled -bg $back
    }\
  else \
    {
    # -- enable the controls
      $options.frameMenu.frameSLat.enLat config -state normal -bg $activeback
      $options.frameMenu.frameSLon.enLon config -state normal -bg $activeback
    }  
  
  }

#-----------------------------------------------------------------------------------#   
# proc MGSBuildList -- fills the MGSEntries array
#-----------------------------------------------------------------------------------#
proc MGSBuildList { } \
   {
   #--set up correct access to the arrays
   global MGSEntries 
   global ServerList ServerListLength

   catch { unset MGSEntries }
   
   
   #--set up correct access to the arrays
   set ServerIndex 0
   
   for { set i 0 } { $i < $ServerListLength } { incr i } \
     {
     if { [string match "G" [lindex $ServerList($i-H) 3] ] } \
       {
       # Have a valid GRID SERVER so add to the list
       set MGSEntries($ServerIndex) [lindex $ServerList($i-H) 0]
       set descriptors [lindex $ServerList($i-H) 7] 
       set EntryIndex 0
       for { set j 0 } { $j<$descriptors } { incr j } \
         {
	 set line $ServerList($i-$j)
	 if { [string match "G" [lindex $line 0 ] ] } \
	   {
           set MGSEntries($ServerIndex,$EntryIndex) [format "%-10s %-20s" [lindex $line 1] [lrange $line 2 end]]
	   incr EntryIndex
	   }
  	 }
       incr ServerIndex	 
       }
     }
   }
#-----------------------------------------------------------------------------------------#
# proc MGSDoList { ServerIndex EntryIndex  } -- get a detailed Grid list at this location #
#	-- ServerIndex -- server to use							  #
#	-- EntryIndex  -- DataSet in the Server to use					  #
#-----------------------------------------------------------------------------------------#
proc MGSDoList { ServerIndex EntryIndex } \
  {
  global MGSEntries
  global enMGSDataSet
  global MGSActiveServer MGSActiveEntry
    
  # -- set the label header here
    set MGSDataSet "WORKING" 
    update
     
  # -- set the Server and Entry correctly
    set MGSActiveServer $ServerIndex
    set MGSActiveEntry $EntryIndex
      
    MGSBuildGridList
      
  # -- set the label header here
     set Group $MGSEntries($MGSActiveServer)
     set enMGSDataSet "$Group/[lindex $MGSEntries($MGSActiveServer,$MGSActiveEntry) 0]"

  }


#-------------------------------------------------------------------------------#
# MGSBuildGridList -- Fills the MGSEntriesList 
#-------------------------------------------------------------------------------#
proc MGSBuildGridList {  } \
   {
   global MGSEntries MGSEntriesList MGSEntriesExist
   global MGSActiveServer MGSActiveEntry
   global MGSParamsList MGSLevelList MGSSourceList MGSProjectionList 
   
   ProgressInit "Building Grid Listing"

   #--- first get the GROUP and DESCRIPTOR
      set Group $MGSEntries($MGSActiveServer)
      set Descriptor [lindex $MGSEntries($MGSActiveServer,$MGSActiveEntry) 0]
      set DescriptorList [open "|grdlist.k $Group/$Descriptor NUM=ALL" r]

   catch { unset MGSEntriesList
           unset MGSEntriesExist }  
      
   #--- burn the first four lines of text and verify logged on as well   
      gets $DescriptorList line	; # burn 1

   #===============================================================#
   #		Verify that the user is logged on
   #===============================================================#
      if { [string match "*Accounting data was not valid*" $line] }  \
      {
         tk_messageBox \
            -title "Server Error" \
            -message "You are not logged on. \nPlease log on and try again"\
            -type ok \
            -parent .tlGridSelect \
            -icon warning

         ProgressClose
         return		
      }

      set Directory [string trim [string range $line 17 20]]
      gets $DescriptorList line	; # burn 2
      gets $DescriptorList line	;# burn 3
      
   # --- initialize index counter
      set MGSIndex 0
   #--- need to eat this line before the loop and loop till done since after eating the bands the "line" will contain valid info
     if { [gets $DescriptorList line] == -1 } \
       {
       catch { close $DescriptorList }
       ProgressClose   
       return
       }
   #-- Lists list of all TYPES of Source/Projection/Parameter/Level
     set MGSSourceList     ""
     set MGSProjectionList "" 
     set MGSParamsList     ""
     set MGSLevelList      ""
   #-- need to track the lengths of each list as well  
     set SLength 0
     set PjLength 0
     set ParLength 0
     set LLength 0  
   #-- suck it all in and build long list 
     while { 1 } \
       {
       # --- go until this line is read and then close and return
       if { [string first "Number of grids listed =" $line]!=-1 }\
	 {
	 while { [gets $DescriptorList line] != -1 } {}
         catch { close $DescriptorList }
         break
	 }
       if { [string first "Dataset position" $line]!=-1 }\
         {
	 #-- need the Directory position number as well
	 set Directory [string trim [string range $line 17 20]]
	 #-- need to burn these three lines
	 gets $DescriptorList line
	 gets $DescriptorList line
	 if { [gets $DescriptorList line] == -1 } { break }
	 }	 
       #-- Lots to filter so lets start
       set Source     [string range $line 38 41]      ;# Source
       set Projection [string range $line 76 79]      ;# Projection
       set Parameter  [string range $line  0  3]      ;# Parameter
       set Level      [string range $line  5 14]      ;# Level
       set Day        [string range $line 16 36]      ;# when it was run
       set Forecast   [string range $line 43 68]      ;# Forecast for
       set Position   [string range $line 70 74]      ;# Position number

       #-- Pad the forecast hour with 0s (3 places) to aid in sorting
       set tmpHour    [string range $Forecast 0 4]    ;# Forecast hour
       set tmpHour    [string trim $tmpHour];
       set preHour    "";
       if { $tmpHour < 100 } { set preHour  "0" }
       if { $tmpHour <  10 } { set preHour "00" }

       #-- Trim the leading whitespace
       set Source     [string trim $Source];
       set Projection [string trim $Projection];
       set Parameter  [string trim $Parameter];
       set Level      [string trim $Level];
       set Day        [string trim $Day];
       set Forecast   [string trim $Forecast];
       set Position   [string trim $Position];

       #-- Add the forecast hour padding
       set Forecast   "$preHour$Forecast"

       #-- Ok generate the index numbers now
       set SIndex [lsearch $MGSSourceList $Source]
       if { $SIndex==-1 } \
         {
	 # This entry wasn't found so add to the list
	 set MGSSourceList [linsert $MGSSourceList $SLength $Source]
	 set SIndex $SLength
	 incr SLength
	 }
       set PjIndex [lsearch $MGSProjectionList $Projection]
       if { $PjIndex==-1 } \
         {
	 set MGSProjectionList [linsert $MGSProjectionList $PjLength $Projection]
	 set PjIndex $PjLength
	 incr PjLength
	 }
       set ParIndex [lsearch $MGSParamsList $Parameter]
       if { $ParIndex==-1 } \
         {
	 set MGSParamsList [linsert $MGSParamsList $ParLength $Parameter]
	 set ParIndex $ParLength
	 incr ParLength
	 }
       set LIndex [lsearch $MGSLevelList $Level]
       if { $LIndex==-1 } \
         {
	 set MGSLevelList [linsert $MGSLevelList $LLength $Level]
	 set LIndex $LLength
	 incr LLength
	 }
       if { [info exists MGSEntriesExist($SIndex,$PjIndex,$ParIndex,$LIndex)] } \
         {
	 incr MGSEntriesExist($SIndex,$PjIndex,$ParIndex,$LIndex)
	 } \
       else \
         {
	 set MGSEntriesExist($SIndex,$PjIndex,$ParIndex,$LIndex) 0
	 }
	 
       set t $MGSEntriesExist($SIndex,$PjIndex,$ParIndex,$LIndex)
       set MGSEntriesList($SIndex,$PjIndex,$ParIndex,$LIndex,$t) [list [list $Day $Forecast] $Position $Directory]
       if { [gets $DescriptorList line] == -1 }  \
         {
         catch { close $DescriptorList }
         break
         }
       incr MGSIndex
       if { ! [expr $MGSIndex % 10] } { ProgressUpdate }
       }    
  # -- for each section of the list -- MGSEntriesList -- we need to pull out the data and sort it
  # -- so that the lists are displayed in sorted order later
     for { set SIndex 0 } { $SIndex<$SLength } { incr SIndex } \
       {
       for { set PjIndex 0 } { $PjIndex<$PjLength } { incr PjIndex } \
         {
	 for { set ParIndex 0 } { $ParIndex<$ParLength } { incr ParIndex } \
	   {
	   for { set LIndex 0 } { $LIndex<$LLength } { incr LIndex } \
	     {
	     if { [info exists MGSEntriesExist($SIndex,$PjIndex,$ParIndex,$LIndex) ] } \
	       {
  	       set t $MGSEntriesExist($SIndex,$PjIndex,$ParIndex,$LIndex) 
	       set PreSortList ""
	       for { set i 0 } { $i < $t } { incr i } \
	         {
	         lappend PreSortList $MGSEntriesList($SIndex,$PjIndex,$ParIndex,$LIndex,$i)
	         }
	       # -- now sort it  
	       set PreSortList [lsort $PreSortList]
	       # -- now put them back into the MGSEntriesList array
	       for { set i 0 } { $i < $t } { incr i } \
	         {
	         set MGSEntriesList($SIndex,$PjIndex,$ParIndex,$LIndex,$i) [lindex $PreSortList $i]
	         }
	       }	 
	     }
	   }
	 }    
       }

  #--Rebuild the menues and fill the lists
    ProgressClose
    MGSSetMenues "SOURCE"
  }
  
#-----------------------------------------------------------------------------------#
# proc MGSClick {  } -- Used to set the DataSet correctly for the Globe
#-----------------------------------------------------------------------------------#
proc MGSClick {  } \
   {
   global MGSLBox MGSGlobe 
   global MGSEntriesList MGSFilteredList MGSActiveServer MGSActiveEntry MGSEntries

   global ListBoxSelections  ; # array of list box entries saved within the MakeListBox proc in CommonProcs.tcl
      
  #--- get list of selection indexes
   set FilterIndex [lindex $ListBoxSelections($MGSLBox) 0]
    
  #--- if none then return  
    if { ! [llength $FilterIndex] } { return }
  
    set Group $MGSEntries($MGSActiveServer)
    set Descriptor [lindex $MGSEntries($MGSActiveServer,$MGSActiveEntry) 0]
  # -- Index built as follows -- 
    set Position [lindex $MGSEntriesList($MGSFilteredList($FilterIndex)) 1]
    set Directory [lindex $MGSEntriesList($MGSFilteredList($FilterIndex)) 2]

   return   
   }
#---------------------------------------------------------------------------#
# proc MGSLoadGrids -- Loads any image with the MGSFlags(x) set to 1 -- only checks under 
#		entries after MGSIndex
#---------------------------------------------------------------------------#
proc MGSLoadGrids { } \
{
   global MGSEntries MGSEntriesList MGSFilteredList
   global MGSDataSet MGSSource MGSProjection MGSParameter MGSLevel
   global MGSActiveServer MGSActiveEntry MGSLBox
   global MGSContourUnits MGSMapOptions MGSContourInterval MGSContourColor MGSDash MGSProjectionSet MGSMapColor
   global MGSLabelInterval MGSLabelHeight MGSBLat1 MGSBLat2 MGSBLon1 MGSBLon2 MGSSLat MGSSLon
   global ListBoxSelections  ; # array of list box entries saved within the MakeListBox proc in CommonProcs.tcl

   #--build up the options here 
   set LoadOptions " MCOLOR=$MGSMapColor COLOR=$MGSContourColor"
   if { ![string match $MGSContourInterval "DEFAULT"] } { append LoadOptions " CINT=$MGSContourInterval" } 
   if { ![string match $MGSContourUnits "DEFAULT"] } { append LoadOptions " UNIT=$MGSContourUnits DASH=$MGSDash" }
   append LoadOptions " LSIZE=$MGSLabelHeight LINT=$MGSLabelInterval"

   #-- MAP= or NAV=C
   if { [string match $MGSMapOptions "NAV=C"] } { append LoadOptions " NAV=C" }\
   else { append LoadOptions " MAP=$MGSMapOptions" }

   #-- Projection string  
   if { ![string match $MGSProjection "DEFAULT"] } \
   {
     if { [string match $MGSProjectionSet "MERC"] } { append LoadOptions " PRO=MERC" } \
     elseif { [string match $MGSProjectionSet "LAMB"] }\
     {
       if { $MGSSLat < 0 } { append LoadOptions " PRO=CONF SLON=$MGSSLon SLAT=-30 -50"} \
       else { append LoadOptions " PRO=CONF SLON=$MGSSLon SLAT=30 50" }
     }\
     elseif { [string match $MGSProjectionSet "PS"] } \
     {
       if { $MGSSLat < 0 } { append LoadOptions " PRO=CONF SLON=$MGSSLon SLAT=-60"} \
       else { append LoadOptions " PRO=CONF SLON=$MGSSLon SLAT=60" }
     }
   } \

   # --- time for the watch
   showBusyStatus
   
   # --- now get each position etc.. and display
   set Group $MGSEntries($MGSActiveServer)
   set Descriptor [lindex $MGSEntries($MGSActiveServer,$MGSActiveEntry) 0]
   
   #set MarkedList [$MGSLBox curselection]
   set MarkedList $ListBoxSelections($MGSLBox)

   set count 1 ;  # grids are already in time order so move UP the list [llength $MarkedList]
   foreach FilterIndex $MarkedList \
   {
     set LoopLoadOptions $LoadOptions

     # --- need the position
     set Position [lindex $MGSEntriesList($MGSFilteredList($FilterIndex)) 1]

     # --- check for use of DERIVED PARAM
     if { [lsearch { "MOVIE" "STREAML" "WINDB" "WINDV" } $MGSParameter ]!=-1 } \
     {
       # need to append some more info here as well
       append LoopLoadOptions " PARAM=$MGSParameter"
       append LoopLoadOptions " LEVEL=[lindex $MGSLevel 0]"
       # now get the day time and forecast hour
         set Temp [lindex $MGSEntriesList($MGSFilteredList($FilterIndex)) 0]
	 set DayStuff [lindex $Temp 0]
	 set ForecastStuff [lindex $Temp 1]
         set JulianDay [lindex $DayStuff 2]
         set Time [lindex $DayStuff 3]
         set FHour [lindex $ForecastStuff 0]    
       append LoopLoadOptions " DAY=$JulianDay"
       append LoopLoadOptions " TIME=$Time"
       append LoopLoadOptions " FHOUR=$FHour"
       set Directory [lindex $MGSEntriesList($MGSFilteredList($FilterIndex)) 2]
       LMLoadGridIntoFrame $Group $Descriptor [string trim $Directory] -9999 $LoopLoadOptions $count
     } \
     elseif { [lsearch {"ABV" "BETA" "COR" "DIR" "DSH" "DST" "DVG" "SPD" "TD" "VOR" } $MGSParameter ]!=-1 } \
     {
       # This is to handle derived parameters
       append LoopLoadOptions " LEVEL=[lindex $MGSLevel 0]"
       # now get the day time and forecast hour
         set Temp [lindex $MGSEntriesList($MGSFilteredList($FilterIndex)) 0]
	 set DayStuff [lindex $Temp 0]
	 set ForecastStuff [lindex $Temp 1]
         set JulianDay [lindex $DayStuff 2]
         set Time [lindex $DayStuff 3]
         set FHour [lindex $ForecastStuff 0]    
       append LoopLoadOptions " DAY=$JulianDay"
       append LoopLoadOptions " TIME=$Time"
       append LoopLoadOptions " FHOUR=$FHour"
       append LoopLoadOptions " DER=$MGSParameter " 
       set Directory [lindex $MGSEntriesList($MGSFilteredList($FilterIndex)) 2]
       LMLoadGridIntoFrame $Group $Descriptor [string trim $Directory] -9999 $LoopLoadOptions $count
     } \
     else \
     {  

       # --- check for invalid GRIB number (indicates GRIB dataset type)
       if { [string match "*N/A*" $Position] }  \
       {
         set Position -9999
         # need to append some more info here as well
         append LoopLoadOptions " PARAM=$MGSParameter"
         append LoopLoadOptions " LEVEL=[lindex $MGSLevel 0]"
         # now get the day time and forecast hour
           set Temp [lindex $MGSEntriesList($MGSFilteredList($FilterIndex)) 0]
	   set DayStuff [lindex $Temp 0]
	   set ForecastStuff [lindex $Temp 1]
           set JulianDay [lindex $DayStuff 2]
           set Time [lindex $DayStuff 3]
         set FHour [lindex $ForecastStuff 0]    
         append LoopLoadOptions " DAY=$JulianDay"
         append LoopLoadOptions " TIME=$Time"
         append LoopLoadOptions " FHOUR=$FHour"
       }

       set Directory [lindex $MGSEntriesList($MGSFilteredList($FilterIndex)) 2]
       LMLoadGridIntoFrame $Group $Descriptor [string trim $Directory] [string trim $Position] $LoopLoadOptions $count 
     }
     incr count 1;  #-1 Grids are already in time order so move up the list (+1 not -1)
   }

   # --- restore cursor
     showIdleStatus      
}   

#-------------------------------------------------------------------------------#
# proc MGSApplyFilter {  } -- builds the MGSFilteredList based on MGSpdate and MGSoffset settings
#	-- LBox -- List box to place entries into
#-------------------------------------------------------------------------------#
proc MGSApplyFilter  { } \
   {
   global MGSEntriesList MGSEntriesExist MGSLBox MGSFilteredList
   global MGSParamsList MGSLevelList MGSSourceList MGSProjectionList 
   global MGSSource MGSProjection MGSLevel MGSParameter MGSListLength
   global MGSjDate 

   # --- if all inclusive set IncludeAll to 1 else 0
      if {$MGSjDate == -1} { set IncludeAll 1} \
      else\
         {   
         # --- set the date
         set date $MGSjDate
         set IncludeAll 0
         }   
         
   # --- Need to clear all of the old entries
      if { [info exists MGSFilteredList] } { unset MGSFilteredList }
   
   # --- Now do the looping and insert any that meet the conditions into the list
     set SIndex   [lsearch $MGSSourceList $MGSSource]
     set PjIndex  [lsearch $MGSProjectionList $MGSProjection]
     
     #-- Special case of Derived Parameters -- trick into thinking using the U parameter
     if { [lsearch { "MOVIE" "STREAML" "WINDB" "WINDV" } $MGSParameter ]!=-1 || \
          [lsearch { "ABV" "BETA" "COR" "DIR" "DSH" "DST" "DVG" "SPD" "TD" "VOR" } $MGSParameter ]!=-1 } \
       {
       set ParIndex [lsearch $MGSParamsList "U"]
       } \
     else  \
       {
       set ParIndex [lsearch $MGSParamsList $MGSParameter]
       }
     set LIndex   [lsearch $MGSLevelList $MGSLevel]

     set Count $MGSEntriesExist($SIndex,$PjIndex,$ParIndex,$LIndex)

   if { ![info exists MGSSourceList] } { return }
   #-- Show Busy status
     showBusyStatus 
   # --- get active line so can reset there if possible
     set LBLine [$MGSLBox nearest 0]
   
   set F 0   
   for { set index 0 } { $index<$Count } { incr index } \
     {
     set prefix [lindex $MGSEntriesList($SIndex,$PjIndex,$ParIndex,$LIndex,$index) 0]
     set Day [lindex $prefix 0]
     set cdate [string range $prefix 8 12]
     if { $IncludeAll || $cdate == $date } \
       {
       set MGSFilteredList($F) "$SIndex,$PjIndex,$ParIndex,$LIndex,$index"
       incr F
       }
     }
  # --  Now go ahead and insert these items into the list -- want in sorted order so build a "pre-list"
  # --  first, sort and then insert into the ListBox
     $MGSLBox delete 0 end
        
     set F 0
     set PreSortList ""
     while { [info exists MGSFilteredList($F)] } \
        {
        set Temp [lindex $MGSEntriesList($MGSFilteredList($F)) 0]
	set DayStuff [lindex $Temp 0]
	set ForecastStuff [lindex $Temp 1]
	$MGSLBox insert $F [format "%s %s" $DayStuff $ForecastStuff]	
        incr F
        }
	
   # -- now reset to position
      $MGSLBox yview $LBLine  
   showIdleStatus   
   }
#---------------------------------------------------------------------------------#
# MGSSetMenues { Level } builds selective menues so that data is always found
#		-- Level -- SOURCE/PROJECTION/PARAMETER/LEVEL
#---------------------------------------------------------------------------------#
proc MGSSetMenues { Level } \
   {
   global MGSMenues
   global MGSEntriesExist MGSEntriesList
   global MGSParamsList MGSLevelList MGSSourceList MGSProjectionList 
   global MGSProjection MGSSource MGSParameter MGSLevel MGSDate MGSjDate


   set f $MGSMenues
   if { [string match CLEAR $Level] } \
     {
     catch { destroy $f.frameSource.mb.menu }
     menu $f.frameSource.mb.menu -tearoff 0
     catch { destroy $f.frameProjection.mb.menu }
     menu $f.frameProjection.mb.menu -tearoff 0
     catch { destroy $f.frameParameter.mb.menu }
     menu $f.frameParameter.mb.menu -tearoff 0
     catch { destroy $f.frameLevel.mb.menu }
     menu $f.frameLevel.mb.menu -tearoff 0
     return
     } \
   elseif { [string match SOURCE $Level] } \
     {
     set buildSource 1
     set buildProjection 1
     set buildParams 1
     set buildLevel 1
     } \
   elseif { [string match PROJECTION $Level] } \
     {
     set buildSource 0
     set buildProjection 1
     set buildParams 1
     set buildLevel  1
     } \
   elseif { [string match PARAMETER $Level] } \
     {
     set buildSource 0
     set buildProjection 0
     set buildParams 1
     set buildLevel  1
     } \
   else \
     {
     set buildSource 0
     set buildProjection 0
     set buildParams 0
     set buildLevel  1
     }  
     
     
   #-- SOURCE -- should only be set when first initializing the menu/list
   if { $buildSource } \
     {
     #--SOURCE
       catch { destroy $f.frameSource.mb.menu }
       set m [menu $f.frameSource.mb.menu -tearoff 0]
       foreach var $MGSSourceList { $m add command -label $var -command "set MGSSource {$var}; MGSSetMenues PROJECTION" }
       set MGSSource [lindex $MGSSourceList 0]
     }
   #-- PROJECTION -- from here on out need to build a new list for valid selections
   #-- lock source to MGSSource and alter all other parameters -- short cut the exit as soon as have a match for that PARAMETER in the list
   if { $buildProjection } \
     {
     catch { destroy $f.frameProjection.mb.menu }
     set m [menu $f.frameProjection.mb.menu -tearoff 0]
     
     set PjLength [llength $MGSProjectionList]
     set PrLength [llength $MGSParamsList]
     set LvLength [llength $MGSLevelList]
     set SIndex [lsearch $MGSSourceList $MGSSource]
     for { set PjIndex 0 } { $PjIndex<$PjLength } { incr PjIndex } \
       {
       for { set PrIndex 0 } { $PrIndex<$PrLength } { incr PrIndex } \
         {
	 for { set LvIndex 0 } { $LvIndex<$LvLength } { incr LvIndex } \
	   {
	   if { [info exists MGSEntriesExist($SIndex,$PjIndex,$PrIndex,$LvIndex)] }\
	     {
	     #-- got all the way through so have a valid selection here
	     #-- go ahead and add to the Projection menu and then set to exit the two inner loops
	     set var [lindex $MGSProjectionList $PjIndex]
             $m add command -label $var -command "set MGSProjection {$var}; MGSSetMenues PARAMETER" 
	     set LVIndex $LvLength
	     set PrIndex $PrLength
	     set MGSProjection $var
	     }
	   }
	 }
       }
     }
   #-- PARAMETER --  need to build a new list for valid selections
   if { $buildParams } \
     {
     catch { destroy $f.frameParameter.mb.menu }
     set m [menu $f.frameParameter.mb.menu -tearoff 0]
     
     set PrLength [llength $MGSParamsList]
     set LvLength [llength $MGSLevelList]
     
     set SIndex [lsearch $MGSSourceList $MGSSource]
     set PjIndex [lsearch $MGSProjectionList $MGSProjection]
     for { set PrIndex 0 } { $PrIndex<$PrLength } { incr PrIndex } \
       {
       for { set LvIndex 0 } { $LvIndex<$LvLength } { incr LvIndex } \
	 {
	 if { [info exists MGSEntriesExist($SIndex,$PjIndex,$PrIndex,$LvIndex)] }\
	   {
	   #-- got all the way through so have a valid selection here
	   #-- go ahead and add to the Projection menu and then set to exit the two inner loops
	   set var [lindex $MGSParamsList $PrIndex]
           $m add command -label $var -command "set MGSParameter {$var}; MGSSetMenues LEVEL"
	   set LvIndex $LvLength
	   set MGSParameter $var
	   }
	 }
       }
     # Now have to insert the four DERIVED Parameters
     foreach pvar { "MOVIE" "STREAML" "WINDB" "WINDV" } \
     {
       $m add command -label $pvar -command "set MGSParameter {$pvar}; MGSSetMenues LEVEL"
     }

     # Now insert values for the DERIVE keyword
     $m add cascade -label "Derived Parameters" -menu $m.derived
     menu $m.derived -tearoff NO
     foreach pvar { "ABV" "BETA" "COR" "DIR" "DSH" "DST" "DVG" "SPD" "TD" "VOR" } \
     {
       $m.derived add command -label $pvar -command "set MGSParameter {$pvar}; MGSSetMenues LEVEL; MGSApplyFilter"
     }
   }
   #-- LEVEL --  need to build a new list for valid selections
   if { $buildLevel } \
     {
     catch { destroy $f.frameLevel.mb.menu }
     set m [menu $f.frameLevel.mb.menu -tearoff 0]

     set LvLength [llength $MGSLevelList]
     
     set SIndex  [lsearch $MGSSourceList $MGSSource]
     set PjIndex [lsearch $MGSProjectionList $MGSProjection]
     #-- Special case of Derived Parameters -- trick into thinking using the U parameter
     if { [lsearch { "MOVIE" "STREAML" "WINDB" "WINDV" } $MGSParameter ]!=-1 || \
          [lsearch { "ABV" "BETA" "COR" "DIR" "DSH" "DST" "DVG" "SPD" "TD" "VOR" } $MGSParameter ]!=-1 } \
       {
       set PrIndex [lsearch $MGSParamsList "U"]
       } \
     else  \
       {
       set PrIndex [lsearch $MGSParamsList $MGSParameter]
       }
     set MGSLevel -1  
     for { set LvIndex 0 } { $LvIndex<$LvLength } { incr LvIndex } \
       {
       if { [info exists MGSEntriesExist($SIndex,$PjIndex,$PrIndex,$LvIndex)] }\
	 {
	 #-- got all the way through so have a valid selection here
	 #-- go ahead and add to the Projection menu and then set to exit the two inner loops
	 set var [lindex $MGSLevelList $LvIndex]
         $m add command -label $var -command "set MGSLevel {$var}; MGSApplyFilter" 
         if { $MGSLevel==-1} { set MGSLevel $var }
	 }
       }
     # when and only when the levels are built -- also build the date range
     set DateIndex 0
     for { set LvIndex 0 } { $LvIndex<$LvLength } { incr LvIndex } \
       {
       if { [info exists MGSEntriesExist($SIndex,$PjIndex,$PrIndex,$LvIndex)] }\
	 {
	 set t $MGSEntriesExist($SIndex,$PjIndex,$PrIndex,$LvIndex)
	 for { set i 0} { $i <= $t } { incr i } \
	   {
	   set a [lindex [lindex $MGSEntriesList($SIndex,$PjIndex,$PrIndex,$LvIndex,$i) 0] 0]
	   set ThisDate [string range  $a 7 11]
           if { ![info exists DateNames($ThisDate)] } \
	    {
	    set DateNames($ThisDate) [getnicedate $ThisDate 0]
  	    set DateList($DateIndex) $ThisDate
	    incr DateIndex
	    }
	   }
         }
       }	 
    #--- finally set up the DATE range stuff   
      catch { destroy $f.frameDate.mb.menu }

    # -- exit early if no Dates arround -- empty dataset
      if { ! [info exists DateList(0)] } \
        { 
        set m [menu $f.frameDate.mb.menu -tearoff 0]
        set MGSDate ""
        return  
        }
   
      set DateIndex 0
   
      set m [menu $f.frameDate.mb.menu -tearoff 0]
      while { [info exists DateList($DateIndex)] } \
        {
        set date $DateNames($DateList($DateIndex))
        $m add command -label $date -command " set MGSDate {$date}
                                               set MGSjDate $DateList($DateIndex)
                                               MGSApplyFilter "
        incr DateIndex     
        }
      $m add command -label "ALL AVAILABLE" -command "set MGSDate {ALL AVAILABLE}; set MGSjDate -1; MGSApplyFilter"
      
      # -- now initialize the Date filter to something   
       set MGSjDate $DateList(0)
       set MGSDate [getnicedate $MGSjDate 0]
     }  
   MGSApplyFilter  
   }  
#------------------------------------------------------------------------------------#
# MGSCountourUnitsMenu { } -- builds the contour units selection
#------------------------------------------------------------------------------------#
proc MGSContourUnitsMenu { f } \
  {
  global MGSCountourUnits 
    
  menubutton $f.mbCU -text "Contour Units" -relief raised -width 16 -borderwidth 2 -indicatoron 1 \
                     -menu $f.mbCU.menu
		     
  menu $f.mbCU.menu -tearoff 0
  $f.mbCU.menu add cascade -label "Linear Measure" -menu $f.mbCU.menu.lin
  $f.mbCU.menu add cascade -label "Pressure"       -menu $f.mbCU.menu.pres
  $f.mbCU.menu add cascade -label "Temperature"    -menu $f.mbCU.menu.temp
  $f.mbCU.menu add cascade -label "Wind"           -menu $f.mbCU.menu.wind
  $f.mbCU.menu add command -label "DEFAULT" -command "set MGSContourUnits DEFAULT"
		     
  #-- Linear Units here
  set m [menu $f.mbCU.menu.lin -tearoff 0]
  $m add radio -label "Centimeters (CM)" -value "CM" -variable MGSContourUnits
  $m add radio -label "Decameters (DM)"  -value "DM" -variable MGSContourUnits
  $m add radio -label "Feet (FT)"        -value "FT" -variable MGSContourUnits
  $m add radio -label "Geopotential Meters (GPM)"    -value "GPM" -variable MGSContourUnits
  $m add radio -label "Inches (IN)"      -value "IN" -variable MGSContourUnits
  $m add radio -label "Kilometers (KM)"      -value "KM" -variable MGSContourUnits
  $m add radio -label "Meters (M)"       -value "M"  -variable MGSContourUnits
  $m add radio -label "Miles (MI)"       -value "MI" -variable MGSContourUnits
  $m add radio -label "Millimeters (MM)" -value "MM" -variable MGSContourUnits
  $m add radio -label "Nautical Miles (NMI)"         -value "NMI" -variable MGSContourUnits
  $m add radio -label "Yards (YD)"       -value "YD" -variable MGSContourUnits

  #-- Pressure Units here
  set m [menu $f.mbCU.menu.pres -tearoff 0]
  $m add radio -label "Inches of Mercury (INHG)" -value "INHG" -variable MGSContourUnits
  $m add radio -label "Hectopascals (HPA)"       -value "HPA"  -variable MGSContourUnits
  $m add radio -label "Millibars (MB)"           -value "MB"   -variable MGSContourUnits
  
  #-- Temperature Units here
  set m [menu $f.mbCU.menu.temp -tearoff 0]
  $m add radio -label "Celsius (C)"     -value "C" -variable MGSContourUnits
  $m add radio -label "Fahrenheit (F)"  -value "F" -variable MGSContourUnits
  $m add radio -label "Kelvin (K)"      -value "K" -variable MGSContourUnits

  #-- Wind Units here
  set m [menu $f.mbCU.menu.wind -tearoff 0]
  $m add radio -label "Knots (KTS)"                -value "KTS" -variable MGSContourUnits
  $m add radio -label "Meters Per Second (MPS)"    -value "MPS" -variable MGSContourUnits
  $m add radio -label "Miles Per Hour (MPH)"       -value "MPH" -variable MGSContourUnits

  #-- now the entry to echo the value
  label $f.entryContourUnits \
	-textvariable MGSContourUnits -width 26 -anchor w -relief sunken -borderwidth 2
  
  #-- pack it
  pack $f.mbCU -side left
  pack $f.entryContourUnits -side left -fill x -expand true
  }
#-------------------------------------------------------------------------------------------#
# MGSMapOptionsMenu {} -- builds the MapOptions menu here
#-------------------------------------------------------------------------------------------#
proc MGSMapOptionsMenu { f } \
  {
  global MGSMapOptions
    
  menubutton $f.mbMO -text "Map Boundaries" -relief raised \
	-width 16 -borderwidth 2 -indicatoron 1 -menu $f.mbMO.menu

  set m [menu $f.mbMO.menu -tearoff 0]
  
  $m add radio -label "Current Navigation" -variable MGSMapOptions -value "NAV=C"
  $m add radio -label "Grid Navigation" -variable MGSMapOptions -value " "
  $m add radio -label "Midwest USA (MID)" -variable MGSMapOptions -value MID 
  $m add radio -label "North America (NA)" -variable MGSMapOptions -value NA 
  $m add radio -label "USA (USA)" -variable MGSMapOptions -value USA 
  $m add cascade -label "States (AL-MT)" -menu $m.states1 
  $m add cascade -label "States (NE-WY)" -menu $m.states2 
  
  set m2 [menu $m.states1 -tearoff 0] 
  foreach state { "Alabama (AL)" "Alaska (AK)" "Arizona (AZ)"\
		"Arkansas (AR)" "California (CA)" "Colorado (CO)" "Connecticut (CT)" \
		"Delaware (DE)" "Florida (FL)" "Georgia (GA)" "Hawaii (HI)" "Idaho (ID)"\
		"Illinois (IL)" "Indiana (IN)" "Iowa (IA)" "Kansas (KS)" "Kentucky (KY)"\
		"Louisiana (LA)" "Maine (ME)" "Maryland (MD)" "Massachusetts (MA)"\
		"Michigan (MI)" "Minnesota (MN)" "Mississippi (MS)" "Missouri (MO)"\
		"Montana (MT)" }\
    {
    set value [string range [lindex $state [expr [llength $state] -1]] 1 2]
    $m2 add radio -label $state -variable MGSMapOptions -value "$value" 
    }		
    
  set m2 [menu $m.states2 -tearoff 0] 
  foreach state {"Nebraska (NE)" "Nevada (NV)" "New Hampshire (NH)"\
		 "New Jersey (NJ)" "New Mexico (NM)" "New York (NY)" "North Carolina (NC)"\
		 "North Dakota (ND)" "Ohio (OH)" "Oklahoma (OK)" "Oregon (OR)"\
		 "Pennsylvania (PA)" "Rhode Island (RI)" "South Carolina (SC)" \
		 "South Dakota (SD)" "Tennessee (TN)" "Texas (TX)" "Utah (UT)" "Vermont (VT)"\
		 "Virginia (VA)" "Washington (WA)" "West Virginia (WV)" "Wisconsin (WI)"\
		 "Wyoming (WY)" } \
    {
    set value [string range [lindex $state [expr [llength $state] -1]] 1 2]
    $m2 add radio -label $state -variable MGSMapOptions -value "$value" 
    }		
		 
    
  label $f.entryMapOptions \
	-textvariable MGSMapOptions \
	-width 26 \
	-anchor w \
	-relief sunken \
	-borderwidth 2
  pack $f.mbMO -side left
  pack $f.entryMapOptions -side left -fill x -expand true
  pack $f -side top -fill x 
  
  }  
#---------------------------------------------------------------------------------------------#
# MGSIntervalMenu -- builds the Contour intervals menu
#---------------------------------------------------------------------------------------------#
proc MGSIntervalMenu { f } \
  {
  global MGSContourInterval
  
  menubutton $f.mbCI -text "Contour Interval" -relief raised -width 16 -borderwidth 2 -indicatoron 1 \
                     -menu $f.mbCI.menu

  set m [menu $f.mbCI.menu -tearoff 0]
  
  foreach interval {DEFAULT 0.05 0.1 0.25 0.5 0.75 1 2 3 4 5 6 7 8 9 10 20 30 40 50 60 70 80 90 100 110 120 130 140 150 } \
    {
    $m add radio -label $interval -variable MGSContourInterval -value "$interval"
    }
  
  label $f.entryContourInterval -textvariable MGSContourInterval -anchor w -relief sunken -borderwidth 2
  pack $f.mbCI -side left
  pack $f.entryContourInterval -side left -fill x -expand true
  }
#---------------------------------------------------------------------------------------------#
# MGSDashMenu -- builds the Dash Options menu
#---------------------------------------------------------------------------------------------#
proc MGSDashMenu { f } \
  {
  global MGSDash
  
  menubutton $f.mbDASH -text "Dash Option" -relief raised -width 16 -borderwidth 2 -indicatoron 1 \
                     -menu $f.mbDASH.menu

  set m [menu $f.mbDASH.menu -tearoff 0]
  
  $m add radio -label "No Dashing (NONE)"   -variable MGSDash -value "NONE"
  $m add radio -label "Negative Only (NEG)" -variable MGSDash -value "NEG"
  $m add radio -label "Positive Only (POS)" -variable MGSDash -value "POS"
  $m add radio -label "All Contours (ALL)"  -variable MGSDash -value "ALL"

  label $f.entryDash -textvariable MGSDash -width 26 -anchor w -relief sunken -borderwidth 2
  pack $f.mbDASH -side left
  pack $f.entryDash -side left -fill x -expand true
  }
#---------------------------------------------------------------------------------------------#
# MGSProjectionMenu -- builds the Projection menu
#---------------------------------------------------------------------------------------------#
proc MGSProjectionMenu { f } \
  {
  global MGSProjectionSet
  
  menubutton $f.mbPJ -text "Projection" -relief raised -width 16 -borderwidth 2 -indicatoron 1 \
                     -menu $f.mbPJ.menu

  set m [menu $f.mbPJ.menu -tearoff 0]
  
  $m add radio -label "Default (DEFAULT)"        -variable MGSProjectionSet -value "DEFAULT" -command { MGSSLatLon 0 }
  $m add radio -label "Polar Stereographic (PS)" -variable MGSProjectionSet -value "PS"      -command { MGSSLatLon 1 }
  $m add radio -label "Lambert Conformal (LAMB)" -variable MGSProjectionSet -value "LAMB"    -command { MGSSLatLon 1 }
  $m add radio -label "Mercator (MERC)"          -variable MGSProjectionSet -value "MERC"    -command { MGSSLatLon 0 }

  label $f.entryProjection -textvariable MGSProjectionSet -anchor w -relief sunken -borderwidth 2
  pack $f.mbPJ -side left
  pack $f.entryProjection -side left -fill x -expand true
  }
#---------------------------------------------------------------------------------------------#
# MGSLabelMenues -- builds the Label menus
#---------------------------------------------------------------------------------------------#
proc MGSLabelMenues { f g } \
  {
  global MGSLabelInterval MGSLabelHeight
  
  menubutton $f.mbLI -text "Label Interval" -relief raised -width 16 -borderwidth 2 -indicatoron 1 \
                     -menu $f.mbLI.menu

  set m [menu $f.mbLI.menu -tearoff 0]
  
  foreach interval { 1 2 3 4 5 6 7 8 9 10 } \
    {
    $m add radio -label $interval -variable MGSLabelInterval -value "$interval"
    }

  label $f.entryLInterval \
	-textvariable MGSLabelInterval -width 26 -anchor w -relief sunken -borderwidth 2
  pack $f.mbLI -side left
  pack $f.entryLInterval -side left -fill x -expand true
  
  
  menubutton $g.mbLH -text "Label Height" -relief raised -width 16 -borderwidth 2 -indicatoron 1 \
                     -menu $g.mbLH.menu

  set m [menu $g.mbLH.menu -tearoff 0]
  
  foreach interval { 0 4 6 8 9 10 11 12 13 14 15 16 17 18 19 20} \
    {
    $m add radio -label $interval -variable MGSLabelHeight -value "$interval"
    }

  label $g.entryLHeight \
	-textvariable MGSLabelHeight -width 26 -anchor w -relief sunken -borderwidth 2
  pack $g.mbLH -side left
  pack $g.entryLHeight -side left -fill x -expand true
  }  
  
