# Copyright(c) 1999, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: ServerListlite.tcl,v 1.13 2001/10/22 17:57:18 tsmith Exp $

##############################################################################
# ADDE Server Lists Module -- keeps DATALOC and some DSINFO information      #
#		so that the ADDE Servers are not queried every time selection#
#		lists are built						     #
##############################################################################
# Global Procedures
#	
#	InitServerLists { } -- Loads data from "SERVERTABLE.TDAT" or creates
#				the file if it doesn't exist
#	SaveServerLists { } -- Saves the ServerTable and all switch settings
#	ServerList	{ } -- Window -- Allows adding and editing of the
#				ServerList
#
# Local Procedures
#
#  	UpdateServerDisplay{}  	-- Used by "ServerList" to keep display current
#	ToggleAccessState{}    	-- Adjusts flag settings for each entry
#       ServerListEntryEdit{}  	-- Allows editing of the ServerList Data    
#       VerifyServerEdit{}     	-- makes sure that the text string contain no spaces
#       DeleteFromServerList{} 	-- Deletes all marked records from the ServerList     
#	CleanServerList{}      	-- Usually needs to be called after records are deleted
#       AddToServerList{} 	-- Adds a record then prompts for editing
#	ResetServerLists{} 	-- does the DSINFO command on each server entry or a 
#				    single specified entry if a position is given
#	SortServerList{} 	-- makes sure table is saved in alphabetical order
#       AddLocalServerEntry{} 	-- inserts a LOCAL_SERVER data entry into the server list					  
#       DeleteLocalServerEntry{}-- removes hilighted LOCAL server entries             
#	EditLocalServerEntry{} 	-- edits a LOCAL_SERVER data entry in the server list
#
# Global Data (Use Externally)
#
#	SeverList -- List of server data, an array of entries as follows
#		ServerList(#-H) -Name IPAdress Flags(I G P T N V) NumberOfDescriptors
#			  (#-#) -Type DescriptorName DiscriptorComment
#			  (#-#) -Type Name Begin End Comment --> For <LOCAL-DATA>
#	ServerListLength -- number of entries in the server list
#
##############################################################################
global ServerList ServerListLength

##############################################################################
# ServerList -- Shows a list of the ADDE servers running in McIDAS           #
##############################################################################
proc ServerList { } \
  {
  global ServerListLength ServerList
  
  if { [winfo exists .tlServer] } { wm deiconify .tlServer; focus .tlServer; return }
  toplevel .tlServer  -relief raised -borderwidth 3
    #------------- Header ------------------------------#
    frame .tlServer.fmHeader -relief raised -borderwidth 3
      MakeListBox .tlServer.fmHeader "        Name                IP Address            Flags     " "" 
      .tlServer.fmHeader.lList config  -width 60 -selectmode extended 
    pack .tlServer.fmHeader -side top -fill x 
    #------------- Options -----------------------------#      
    frame .tlServer.fmOptions -relief raised -borderwidth 3
      frame .tlServer.fmOptions.fm1
        button .tlServer.fmOptions.fm1.btEdit -text "Edit" -highlightthickness 0  \
           -command { ServerListEntryEdit [.tlServer.fmHeader.lList curselection]  }
        button .tlServer.fmOptions.fm1.btDelete -text "Delete" -highlightthickness 0  \
           -command { set MarkedToDelete [ .tlServer.fmHeader.lList curselection ] ;  \
	              foreach i $MarkedToDelete { DeleteFromServerList $i  } ; \
		      CleanServerList ; \
                      UpdateServerDisplay }
        button .tlServer.fmOptions.fm1.btAdd -text "Add ADDE Server" -highlightthickness 0  \
           -command { AddToServerList "" }
        button .tlServer.fmOptions.fm1.btAddLocal -text "Add Local Data" -highlightthickness 0 \
           -command { AddToServerList Local }
		      
        pack .tlServer.fmOptions.fm1.btEdit  -side left -padx 15
        pack .tlServer.fmOptions.fm1.btDelete -side left -padx 15
        pack .tlServer.fmOptions.fm1.btAdd -side left -padx 5 
        pack .tlServer.fmOptions.fm1.btAddLocal -side left -padx 5
      pack .tlServer.fmOptions.fm1 -side top -fill x -pady 5
      frame .tlServer.fmOptions.fm2
        button .tlServer.fmOptions.fm2.btRegen -text "Rebuild ADDE List" -highlightthickness 0\
	   -command  { if { ![GUILogon "Continue"] } \
	                 {
	                 ResetServerLists " " 
	                 UpdateServerDisplay 
			 }
		     }	 
        button .tlServer.fmOptions.fm2.btExit -text "Exit" -highlightthickness 0  \
           -command { destroy .tlServer 
                      # -- Save the data as its altered
                        SaveServerLists
                    }
        pack .tlServer.fmOptions.fm2.btRegen -side left -padx 15
        pack .tlServer.fmOptions.fm2.btExit -side right -padx 15      
      pack .tlServer.fmOptions.fm2 -side top -fill x -pady 5
    pack .tlServer.fmOptions -side top -fill x -ipady 10

  bind .tlServer.fmHeader.lList <Double-1> { ServerListEntryEdit [.tlServer.fmHeader.lList index active] }
  bindtags .tlServer.fmHeader.lList [list Listbox .tlServer.fmHeader.lList . all ]
  bind .tlServer  <Destroy> { catch { destroy .tlServerEntryEdit } 
                              catch { destroy .tlLocalEdit }    
                            }
   
  wm title .tlServer "Server List"
  UpdateServerDisplay
  
  
  return  
  }
    
##############################################################################
# InitServerLists -- Initializes array that holds data on the server lists   #
#		This procedure will load the data from file.  If no file is  #
#		found, then a DATALOC will be run as well as a DSINFO and a  #
#		file will be generated.					     #
##############################################################################
proc InitServerLists {  } \
  {
  global ServerList ServerListLength
  
  catch { unset ServerList }
  set ServerListLength 0
  
  set fname [GetLocalDataPath SERVERTABLE.TDAT]

  if { [file exists $fname] } \
    {
    if [ catch {open $fname r} fileID] \
      {
      # error message if you want one
      return
      }\
    else \
      {
      set ServerListLength 0
      while { [gets $fileID line] > -1 } \
        {
	set ServerList($ServerListLength-H) $line
	set Descriptors [lindex $line 8]
	set i 0
	while { $i < $Descriptors } \
	  {
	  gets $fileID ServerList($ServerListLength-$i)
	  incr i
	  }
	incr ServerListLength  
	}
      catch { close $fileID }
      }   
    }  \
  else \
    {  
    #---following is a message to warn first time loaders that this may take a while
#ml    set retval [AcceptListReset]
#ml    if { $retval } { wm withdraw .
#ml    		     if { ![GUILogon "Continue"] } { ResetServerLists " " }
#ml		       wm deiconify . 
#ml		   }
    }
  return
  }    
##############################################################################
# SaveServerLists -- Saves the server list data as it exists now             #
##############################################################################
proc SaveServerLists { } \
  {
  global ServerList ServerListLength
  
  #-- don't save the data unless there is something to save!!!
    if { $ServerListLength==0 } { return }
    
  #---OK now open a file "SERVERTABLE.TDAT" and save the ServerTable
  set fname [GetLocalDataPath SERVERTABLE.TDAT]
  if [ catch { open $fname w } fileID ] \
    {
    # error message if you want one
    puts stdout "couldn't open the SERVERTABLE.TDAT file"
    return
    }\
  else \
    {
    for { set i 0 } { $i < $ServerListLength } { incr i } \
      {
      puts $fileID $ServerList($i-H)
      set Descriptors [lindex $ServerList($i-H) 8]
      for { set j 0 } { $j < $Descriptors } { incr j } \
        {
        puts $fileID $ServerList($i-$j)
	}
      }
    catch { close $fileID }
    }  
  return
  }    
##############################################################################
# UpdateServerDisplay -- Places the server NAME, IP_ADDRESS and flags info   #
#		into the ServerList window				     #
##############################################################################
proc UpdateServerDisplay { }\
  {
  global ServerList ServerListLength
  
  # --- make sure that the parent exists -- else exit
    if { ![winfo exists .tlServer] } { return }

  #--- clear the entire list box
    .tlServer.fmHeader.lList delete 0 end
  
  #--- Insert each line
  for { set i 0 } { $i < $ServerListLength } { incr i } \
    {
    set a [format "%-20s %-25s %s" [lindex $ServerList($i-H) 0] \
                                   [lindex $ServerList($i-H) 1] \
                                   [lrange $ServerList($i-H) 2 7]]
    .tlServer.fmHeader.lList insert end $a
    }
  }  
##############################################################################
# ResetServerLists -- Uses DATALOC and DSINFO to rebuild the entire list     #
#	              if matches are found, the flag settings are saved      #
##############################################################################
proc ResetServerLists { position } \
  {
  # start -- start of dsinfo loop
  # end   -- end of dsinfo loop
  
  global ServerList ServerListLength
  
  #-- A "Not Logged On" warning window will appear if list is rebuilt and not logged on
  set WarnWindow 0
  
  ProgressInit "Building ADDE Server Lists"
  
  #--- if position is set (0-n) 
  if { ![string match $position " "] } \
    {
    # -- set start and end ocunters so that DSINFO loop below knows what to reset
    set start $position
    set end [expr $position + 1]
    #---now clear out this entry completely
    set i 0
    while { [info exists ServerList($position-$i)] } \
      {
      unset ServerList($position-$i)
      incr i
      }
    } \
  else \
    {
    #------  do a DATALOC
      set FileID [open "|dataloc.k" r]   
      set ServerListLength 0

    # -- eat the first three lines
      gets $FileID line
      gets $FileID line
      gets $FileID line
      while { [gets $FileID line] > -1 } \
        {
        if { [string length [lindex $line 1]] == 0 } \
          {
          #--- DONE with the list of servers -- eat the rest of the lines and exit
          while { [gets $FileID line] > -1 } {}
          break 
          }
        set ServerList($ServerListLength-H) "[lindex $line 0] [lindex $line 1] "
        incr ServerListLength
	ProgressUpdate
        }
      catch { close $FileID }
    #--- need to do a DSINFO on all of them
      set start 0
      set end $ServerListLength
    }
  #----- step two is a DSINFO on each set to get its attributes
  #----- or a DSSERVE if it's LOCAL data
  for { set i $start } { $i < $end } { incr i } \
    {
    set Descriptors 0
    # --- reset to just it's name and IP address with empty flags 
    set ServerList($i-H) "[lrange $ServerList($i-H) 0 1] - - - - - -"
    # --- if it's local data do just that first
    if { [string match [lindex $ServerList($i-H) 1] "<LOCAL-DATA>"] } \
      {
      set FileID [open "|dsserve.k LIST [lindex $ServerList($i-H) 0]" r]
      gets $FileID line
      if { [string first "No" $line]>-1 || \
           [string first "Accounting data was not valid" $line]>-1} \
        {
        if { [string first "Accounting data was not valid" $line]>-1} { set WarnWindow 1 }
        #---burn em up --- no data is being returned
        while { [gets $FileID line] > -1 } {}
        #---Set the flag as not present "-"
  	set ServerList($i-H) "$ServerList($i-H) - - - - - -"
        }\
      else \
        {
	#---- burn 2 more lines
	gets $FileID line
	gets $FileID line
        while { [gets $FileID line] > -1 } \
	  {
	  ProgressUpdate
          if { [string first $line "dsserve.k: done"]==-1 } \
	    {
	    #--- reading in valid data
            set type [string range $line 25 29]
            set comment [string range $line 53 end]
	    set name [lindex [split [lindex $line 0] / ] 1]
	    set beginnum [lindex [split [string range $line 36 48] - ] 0]
	    set endnum [lindex [split [string range $line 36 48] - ] 1]

            #--- set the flag value correctly
            switch -- [string range $type 0 0] \
              {
              I { set pos 2 }
              G { set pos 3 }
	      P { set pos 4 }
	      T { set pos 5 }
	      N { set pos 6 }
              }
	    if { ![string length $name]} \
	      {
	      # just ate a line that is realing a description continuation (COMMENT) 
	      incr Descriptors -1
	      set ServerList($i-$Descriptors) [format "%s%s" $ServerList($i-$Descriptors) $comment]
	      } \
	    else \
	      {
  	      set ServerList($i-H) [lreplace $ServerList($i-H) $pos $pos [string range $type 0 0] ] 
  	      set ServerList($i-$Descriptors) [format "%s %-10s   %s %s    %s" [string range $type 0 0] \
	      	  $name $beginnum $endnum $comment ]
	      }
            incr Descriptors
            } \
	  else\
	    {
	    #--- burn it up
            while { [gets $FileID line] > -1 } {}
	    break;  
	    }
	  }
	}  
      catch { close $FileID }
      }\
    else \
      {
      # --- reset to just it's name and IP address
      set ServerList($i-H) [lrange $ServerList($i-H) 0 1]
      foreach type { "IMAGE" "GRID" "POINT" "TEXT" "NAV" "VPROF" } \
        {
        #---- SERVER NAME
        set FileID [open "|dsinfo.k $type [lindex $ServerList($i-H) 0]" r]
        gets $FileID line
        if { [string first "No" $line]>-1 || \
             [string first "Accounting data was not valid" $line]>-1} \
          {
          if { [string first "Accounting data was not valid" $line]>-1} { set WarnWindow 1 }
          #---burn em up --- no data is being returned
          while { [gets $FileID line] > -1 } {}
	  #---Set the flag as not present "-"
  	  set ServerList($i-H) "$ServerList($i-H) -"
	  }\
        else \
          {
	  #---Set the flag as present and active "I/G/P etc.."
	  set ServerList($i-H) [format "%s %s" $ServerList($i-H) [string range $type 0 0]]
          #---burn 4 lines from the dsinfo --- header information
          gets $FileID line
          gets $FileID line
          gets $FileID line
          gets $FileID line
          while { [gets $FileID line] > -1 } \
            {
	    ProgressUpdate
	    if { [string length [lindex $line 1]] } \
	      {
  	      set ServerList($i-$Descriptors) [format "%s %-10s %s" [string range $type 0 0] \
	              [lindex $line 0] [lrange $line 2 end]]
	      incr Descriptors
	      } \
	    else\
	      {
              while { [gets $FileID line] > -1 } {}
	      break;  
	      }
	    }  
	    
	  }
        catch { close $FileID }
	}
      }  
    set ServerList($i-H) "$ServerList($i-H) $Descriptors"  
    }
  # -- Save the data as its altered
    SaveServerLists
  
  ProgressClose  
  if { $WarnWindow } \
    {
    tk_messageBox -title "Server Lists" -message "You were not logged on.  Server entries may be incomplete." \
         		-type ok -parent . -icon warning

    }
  }

##############################################################################
# DeleteFromServerList -- Deletes all marked records from the ServerList     #
##############################################################################
proc DeleteFromServerList { i } \
  {
  global ServerList ServerListLength
  
  # -- OK wipe it out    
    set name [lindex $ServerList($i-H) 0]
    set ip [lindex $ServerList($i-H) 1]
    if { [string match $ip "<LOCAL-DATA>" ] } \
      {
      # -- need to remove with the DSSERVE command
      set j 0
      while { [info exists ServerList($i-$j)] } \
        {
	catch { exec dsserve.k DEL $name/[lindex $ServerList($i-$j) 1] }
        incr j
        }
      }
    catch { exec dataloc.k DEL $name }
    unset ServerList($i-H)   
    set j 0
    while { [info exists ServerList($i-$j)] } \
      {
      unset ServerList($i-$j)
      incr j
      }
  }
##############################################################################
# CleanServerList -- Makes sure that all entries are contiguous              #
##############################################################################
proc CleanServerList { } \
  {
  global ServerList ServerListLength
  
  #---Now we need to clean up the list so it is still a valid arraya
  for { set i 0 } { $i < $ServerListLength } { incr i } \
    {
    if { ! [info exists ServerList($i-H) ] } \
      {
      # found our first empty entry
      # now find the next full entry and replace 
      for { set j $i } { $j < $ServerListLength } { incr j } \
        {
	if { [info exists ServerList($j-H) ] } \
	  {
	  # got one so copy it all down and delete it as we go
	  # HEADER
	    set ServerList($i-H) $ServerList($j-H)
	    unset ServerList($j-H)
	  # DESCRIPTORS 
	    set k 0
	    while { [info exists ServerList($j-$k)] } \
	      {
	      set ServerList($i-$k) $ServerList($j-$k)
	      unset ServerList($j-$k)
	      incr k
	      }
	  #---force the exit condition    
  	    set j $ServerListLength
	  }  
	}
      }
    }
  #--- update the entry total in the ServerList -----  
    set ServerListLength 0
    while { [info exists ServerList($ServerListLength-H)] } { incr ServerListLength }
  }
##############################################################################
# SortServerList -- Makes sure that all entries are in alphabetical order    #
##############################################################################
proc SortServerList { } \
  {
  global ServerList ServerListLength
  
  
  #--- not efficient but quick to code
  set n [expr $ServerListLength - 1 ]
  
  while { $n } \
    {
    set flag 0
    for { set j 0 } { $j < $n } { incr j } \
      {
      set k [expr $j + 1]
      if { [string compare $ServerList($j-H) $ServerList($k-H)] == 1 }\
        {
	set flag 1
	#--- swap them
	catch { unset temp }
	set temp(0-H) $ServerList($j-H)
	set ServerList($j-H) $ServerList($k-H)
	set l 0
	while { [info exists ServerList($j-$l)] } \
	  { 
	  set temp(0-$l) $ServerList($j-$l)
	  unset ServerList($j-$l)
	  incr l
	  }
	set l 0
	while { [info exists ServerList($k-$l)] } \
	  {
	  set ServerList($j-$l) $ServerList($k-$l)
	  unset ServerList($k-$l)
	  incr l
	  }
	set ServerList($k-H) $temp(0-H)  
	set l 0
	while { [info exists temp(0-$l)] } \
	  {
	  set ServerList($k-$l) $temp(0-$l)
	  incr l
	  }
	unset temp  
	}
      }
    incr n -1
    if { ! $flag}  { return }
    }
  }

##############################################################################
# AddToServerList -- Add a record to the server list and then sets its flags #
#		  -- only used as a front end to ServerListEntryEdit         #
##############################################################################
proc AddToServerList { type } \
  {
  global ServerList ServerListLength
  
  if { [string match $type "Local"] } \
    {
    set ServerList($ServerListLength-H) "NAME_TO_ADD <LOCAL-DATA> - - - - - - 0" 
    }\
  else\
    {
    set ServerList($ServerListLength-H) "NAME_TO_ADD IP_ADDRESS - - - - - - 0" 
    }
  incr ServerListLength
  ServerListEntryEdit [expr $ServerListLength -1] 
  }
  

##############################################################################
# ServerListEntryEdit { postion }-- Allows editing of the ServerList Data    #
#	-- position -- data entry to edit				     #
##############################################################################
proc ServerListEntryEdit { position } \
  {
  global ServerList
  global ServerName ServerIPAdd
  global tposition
  
  # -- make sure window is gone first
    catch { destroy .tlServerEntryEdit }    

  set ServerName [lindex $ServerList($position-H) 0]
  set ServerIPAdd [lindex $ServerList($position-H) 1]
  set tposition $position  ;# keep track of position on entry

  set LocalData [string match $ServerIPAdd "<LOCAL-DATA>"] 

  toplevel .tlServerEntryEdit -relief raised -borderwidth 3
  wm title .tlServerEntryEdit "Edit ADDE Entry"

  frame .tlServerEntryEdit.fmHeader -relief flat 
    label .tlServerEntryEdit.fmHeader.lbName -text "ADDE Name" -width 20
    if { !$LocalData } \
      { 
      label .tlServerEntryEdit.fmHeader.lbIPAdd -text "IP Address" -width 25 
      }
    label .tlServerEntryEdit.fmHeader.lbFlags -text "Data Types" -width 20
    pack .tlServerEntryEdit.fmHeader.lbName -side left -pady 5 -padx 10 
    if { !$LocalData } \
      { 
      pack .tlServerEntryEdit.fmHeader.lbIPAdd -side left -pady 5 -padx 10 
      }
    pack .tlServerEntryEdit.fmHeader.lbFlags -side left -pady 5 -padx 10 
  pack .tlServerEntryEdit.fmHeader -side top -fill x 
  
  frame .tlServerEntryEdit.fmEntry -relief flat
    entry .tlServerEntryEdit.fmEntry.enName -textvariable ServerName -width 20 \
       -background white -highlightthickness 0 -relief sunken
    if { !$LocalData } \
      { 
      entry .tlServerEntryEdit.fmEntry.enIPAdd -textvariable ServerIPAdd -width 25 \
             -background white -highlightthickness 0 -relief sunken
      bind .tlServerEntryEdit.fmEntry.enIPAdd <KeyPress> { set ServerIPAdd [string toupper $ServerIPAdd ] }
      bindtags .tlServerEntryEdit.fmEntry.enIPAdd [list Entry . .tlServerEntryEdit.fmEntry.enIPAdd all ]
      }
  
    bind .tlServerEntryEdit.fmEntry.enName <KeyPress> { set ServerName [string toupper $ServerName ] }
    bindtags .tlServerEntryEdit.fmEntry.enName [list Entry . .tlServerEntryEdit.fmEntry.enName all ]

    frame .tlServerEntryEdit.fmEntry.fmFlags 
      set a .tlServerEntryEdit.fmEntry.fmFlags    
      #-- flags/types
      button $a.btImage -text "I" -font smallfont -width 1 -highlightthickness 0 \
       			-foreground black -command "ToggleAccessState $a.btImage $position 2"
      button $a.btGrid  -text "G" -font smallfont -width 1 -highlightthickness 0 \
      			-foreground black -command "ToggleAccessState $a.btGrid $position 3"
      button $a.btPoint -text "P" -font smallfont -width 1 -highlightthickness 0 \
      			-foreground black -command "ToggleAccessState $a.btPoint $position 4"
      button $a.btText  -text "T" -font smallfont -width 1 -highlightthickness 0 \
      			-foreground black -command "ToggleAccessState $a.btText $position 5"
      button $a.btNav   -text "N" -font smallfont -width 1 -highlightthickness 0 \
      			-foreground black -command "ToggleAccessState $a.btNav $position 6"
      button $a.btVProf -text "V" -font smallfont -width 1 -highlightthickness 0 \
      			-foreground black -command "ToggleAccessState $a.btVProf $position 7"

      pack $a.btImage -side left
      pack $a.btGrid  -side left
      pack $a.btPoint -side left
      pack $a.btText  -side left
      pack $a.btNav   -side left
      pack $a.btVProf -side left
    pack .tlServerEntryEdit.fmEntry.enName  -side left -padx 10
    if { !$LocalData } \
      { 
      pack .tlServerEntryEdit.fmEntry.enIPAdd -side left -padx 10
      }
    pack $a -side left -padx 10
  pack .tlServerEntryEdit.fmEntry -side top -fill x 

  frame .tlServerEntryEdit.fmList
    MakeListBox .tlServerEntryEdit.fmList "" ""     
    bind .tlServerEntryEdit.fmList.lList <Double-1> { EditLocalServerEntry $tposition [.tlServerEntryEdit.fmList.lList index active]} 
    bindtags .tlServerEntryEdit.fmList.lList [list Listbox .tlServerEntryEdit.fmList.lList . all ]
  pack .tlServerEntryEdit.fmList -side top -padx 5 -pady 5 -fill x -expand true

  #--config settings here
    SetFlags $position 
  
  frame .tlServerEntryEdit.fmOptions -borderwidth 2
    if { !$LocalData } \
      {
      button .tlServerEntryEdit.fmOptions.btOK     -text "Get Dataset Names" \
       -command "VerifyServerEdit $position"
      }\
    else \
      {
      button .tlServerEntryEdit.fmOptions.btAddEntry -text "Add Local Entry" \
        -command "AddLocalServerEntry $position"
      button .tlServerEntryEdit.fmOptions.btEditEntry -text "Edit Local Entry" \
        -command { EditLocalServerEntry $tposition [.tlServerEntryEdit.fmList.lList index active] }
      button .tlServerEntryEdit.fmOptions.btRemoveEntry -text "Delete Local Entry" \
        -command "DeleteLocalServerEntry $position"
      }   
    button .tlServerEntryEdit.fmOptions.btCancel -text "Exit"  -width 10 \
       -command { destroy .tlServerEntryEdit; \
                  if { [string match $ServerName "NAME_TO_ADD"] || [string match $ServerIPAdd "IP_ADDRESS"] } \
		     { DeleteFromServerList $tposition; CleanServerList } \
                }
    if { !$LocalData } \
      {
      pack .tlServerEntryEdit.fmOptions.btOK -side left -padx 10 -pady 5
      }\
    else \
      {
      pack .tlServerEntryEdit.fmOptions.btAddEntry -side left -padx 10 -pady 5
      pack .tlServerEntryEdit.fmOptions.btEditEntry -side left -padx 10 -pady 5
      pack .tlServerEntryEdit.fmOptions.btRemoveEntry -side left -padx 10 -pady 5
      }  
    pack .tlServerEntryEdit.fmOptions.btCancel -side right -padx 10 -pady 5
  pack .tlServerEntryEdit.fmOptions -side top -pady 5 -fill x -expand true 
  #--- kill children on exit
    bind .tlServerEntryEdit <Destroy> { catch { destroy .tlLocalEdit } }

  #--- force this window active first
    raise .tlServerEntryEdit
  #--- Now Add an entry immediately if local data    
    if { $LocalData && [string match $ServerName "NAME_TO_ADD"] } { AddLocalServerEntry $position }
    
  tkwait window .tlServerEntryEdit
  SortServerList
  UpdateServerDisplay
  }
#-----------------------------------------------------------------------------------#
# VerifyServerEdit{} -- Attaches IP Address to Server Entry (dataloc.k) and updates #
# 			the flags and ServerList				    #
#-----------------------------------------------------------------------------------#
proc VerifyServerEdit { position } \
  {
  global ServerList ServerListLength
  global ServerName ServerIPAdd
  
  
  if { [string length [lindex $ServerName 1]] != 0 } \
    {
    #-- check for spaces -- if more than one item in the "list" i.e. a space then failed
    tk_messageBox -title "Server Name Error" -message "The Server name can not contain spaces." \
         		-type ok -parent . -icon warning
    } \
  elseif  { [string length [lindex $ServerIPAdd 1]] != 0 } \
    {
    #-- check for spaces -- if more than one item in the "list" i.e. a space then failed
    tk_messageBox -title "IP Address Error" -message "The IP Address can not contain spaces." \
         		-type ok -parent . -icon warning
    } \
  else \
    { 
    #--- need to delete what's there now
    DeleteFromServerList $position 
    set ServerList($position-H) [format "%-20s %-25s" $ServerName $ServerIPAdd]
    # --- add a new server    
    catch { exec dataloc.k ADD $ServerName $ServerIPAdd }
    ResetServerLists $position
    #----Now need to reset the data flags
    SetFlags $position 
    }
  }
proc SetFlags { position } \
  {
  global ServerList
  
  #--- parent may have been destroyed so exit if that's the case
    if { ![winfo exists .tlServerEntryEdit] } { return}
  set W .tlServerEntryEdit.fmEntry.fmFlags
  set L .tlServerEntryEdit.fmList.lList
  
  $W.btImage config -state normal -relief raised -background gray90
  switch -- [lindex $ServerList($position-H) 2] \
    {
    - { $W.btImage config -state disabled -relief flat}	
    * { $W.btImage config -background "#990000" -activebackground red}
    I { $W.btImage config -background "#009900" -activebackground green}
    }
    
  $W.btGrid config -state normal -relief raised -background gray90
  switch -- [lindex $ServerList($position-H) 3] \
    {
    - { $W.btGrid config -state disabled -relief flat}
    * { $W.btGrid config -background "#990000" -activebackground red}
    G { $W.btGrid config -background "#009900" -activebackground green}
    }
  $W.btPoint config -state normal -relief raised -background gray90
  switch -- [lindex $ServerList($position-H) 4] \
    {
    - { $W.btPoint config -state disabled -relief flat}
    * { $W.btPoint config -background "#990000" -activebackground red}
    P { $W.btPoint config -background "#009900" -activebackground green}
    }
  $W.btText config -state normal -relief raised -background gray90
  switch -- [lindex $ServerList($position-H) 5] \
    {
    - { $W.btText config -state disabled -relief flat}
    * { $W.btText config -background "#990000" -activebackground red}
    T { $W.btText config -background "#009900" -activebackground green}
    }
  $W.btNav config -state normal -relief raised -background gray90
  switch -- [lindex $ServerList($position-H) 6] \
    {
    - { $W.btNav config -state disabled -relief flat}
    * { $W.btNav config -background "#990000" -activebackground red}
    N { $W.btNav config -background "#009900" -activebackground green}
    }
  $W.btVProf config -state normal -relief raised -background gray90
  switch -- [lindex $ServerList($position-H) 7] \
    {
    - { $W.btVProf config -state disabled -relief flat}
    * { $W.btVProf config -background "#990000" -activebackground red}
    V { $W.btVProf config -background "#009900" -activebackground green}
    }

  #------Now fill the list box-----    
  set length [lindex $ServerList($position-H) 8]
  
  $L delete 0 end
  for { set i 0} { $i < $length } { incr i} \
    {
    $L insert end $ServerList($position-$i)
    }
    
  }  

proc ToggleAccessState { w Number Index } \
  {
  global ServerList
    
  set CompTo [lindex $ServerList($Number-H) $Index]
  set SetTo *
  switch -- $Index \
    {
    2 { if {[string match "\\*" $CompTo] } { set SetTo I } }
    3 { if {[string match "\\*" $CompTo] } { set SetTo G } }
    4 { if {[string match "\\*" $CompTo] } { set SetTo P } }
    5 { if {[string match "\\*" $CompTo] } { set SetTo T } }
    6 { if {[string match "\\*" $CompTo] } { set SetTo N } }
    7 { if {[string match "\\*" $CompTo] } { set SetTo V } }
    }

  if { [string match "\\*" $SetTo] } \
    { 
    $w config -background "#bb0000" -activebackground red
    } \
  else \
    { 
    $w config -background "#009900" -activebackground green
    }  
  
  set ServerList($Number-H) [lreplace $ServerList($Number-H) $Index $Index $SetTo]
  }

#-------------------------------------------------------------------------#
# AddLocalServerEntry { position } -- inserts a LOCAL_SERVER data entry   #
#		into the server list					  #
#-------------------------------------------------------------------------#
proc AddLocalServerEntry { position } \
  {
  global ServerList ServerListLength
  global ServerName ServerIPAdd
 
  if { [string length [lindex $ServerName 1] ] != 0 } \
    {
    #-- check for spaces -- if more than one item in the "list" i.e. a space then failed
    tk_messageBox  -title "ADDE Error" -message "DataSet name can contain no spaces." \
                   -type ok -parent .  -icon warning   
    } \
  else \
    { 
    #--- now see if the name has changed from something other than NAME_TO_ADD 
    #--- if so then need to convert the entire existing list
    set length [lindex $ServerList($position-H) 8]
    set OldName [lindex $ServerList($position-H) 0]
    if { [string match $OldName "NAME_TO_ADD"]  || [string match $OldName $ServerName] } \
      {
      # --- no problem -- just replace the name part correctly cuz it's all new (no list to change)
      } \
    else \
      {
      # --- need to rename the entire set of server data to the new name
      for { set i 0 } { $i < $length } { incr i } \
        {
	catch { exec dsserve.k DEL $OldName/[lindex $ServerList($position-$i) 1] }

        set Type [lindex $ServerList($position-$i) 0]
        set Name [lindex $ServerList($position-$i) 1]
	set Begin [lindex $ServerList($position-$i) 2]
	set End [lindex $ServerList($position-$i) 3]
        set Comment [lrange $ServerList($position-$i) 4 end]
	
        catch { exec dsserve.k ADD $ServerName/$Name X $Begin $End TYPE=$Type \"$Comment }
	}
      catch { exec dataloc.k DEL $OldName }	
      }      
    #--- Build a new entry in the list so it can be edited
    set ServerList($position-$length) "I ???? 0 100"
    incr length
    set ServerList($position-H) [format "%s %s %s" $ServerName [lrange $ServerList($position-H) 1 7] $length]
    EditLocalServerEntry $position [expr $length -1]
    }
  }  
#-------------------------------------------------------------------------#
# EditLocalServerEntry { position offset } -- edits a LOCAL_SERVER data   #
#		entry in the server list			 	  #
#-------------------------------------------------------------------------#
proc EditLocalServerEntry { position offset } \
  {
  global ServerList ServerListLength
  global LocalType LocalName LocalComment LocalBegin LocalEnd LocalFormat
  global KeepFlag
  global tServerName ServerName

  # -- chance that it's being called an empty list
  if { ![info exists ServerList($position-$offset)] } { return }

  # -- make sure window is gone first
    catch { destroy .tlLocalEdit }    

  set LocalType [lindex $ServerList($position-$offset) 0]
  set LocalName [lindex $ServerList($position-$offset) 1]
  set LocalBegin [lindex $ServerList($position-$offset) 2]
  set LocalEnd [lindex $ServerList($position-$offset) 3]
  set LocalComment [lrange $ServerList($position-$offset) 4 end]
  
  set KeepFlag  [expr ! [ string match $LocalName "????" ] ]

  set NewServerName [string match $ServerName "NAME_TO_ADD"]
  set tServerName $ServerName
  
  set RemoveName $LocalName ;# if the name changes-- then will need to remove this one
  if {$LocalBegin==0 } { set LocalBegin 1 } ;# on init it may be zero but that's invalid so set to 1

  #--- expand LocalType so it's legible
  switch -- [string range $LocalType 0 0 ] \
    {
    I {
      set LocalType "IMAGE"
      set FormatList "{AREA} {NCDF}"
      }
    G {
      set LocalType "GRID" 
      set FormatList "{GRID} {NCDF}"
      }
    P {
      set LocalType "POINT"
      set FormatList "MD"
      }
    T {
      set LocalType "TEXT" 
      set FormatList "TEXT"
      }
    N {
      set LocalType "NAV"  
      set FormatList "SYSN"
      }
    }

  set LocalFormat [lindex $FormatList 0]
  #--- build the entry dialog
  toplevel .tlLocalEdit
    wm title .tlLocalEdit "Local Data Description"
    frame .tlLocalEdit.fmMain -relief raised -borderwidth 3
      #--- top row of entries -- name/comment
      frame .tlLocalEdit.fmMain.fmTop 
 	label .tlLocalEdit.fmMain.fmTop.lbServer -text "             Group               Descriptor" -anchor w
        pack .tlLocalEdit.fmMain.fmTop.lbServer -side top
        frame .tlLocalEdit.fmMain.fmTop.fmName
          label .tlLocalEdit.fmMain.fmTop.fmName.lbName -text "Dataset Name:"
	  entry .tlLocalEdit.fmMain.fmTop.fmName.enServerName -textvariable tServerName -background white
      
          bind .tlLocalEdit.fmMain.fmTop.fmName.enServerName <KeyPress> { set tServerName [string toupper $tServerName ] }
          bindtags .tlLocalEdit.fmMain.fmTop.fmName.enServerName [list Entry . .tlLocalEdit.fmMain.fmTop.fmName.enServerName all]
	  # if the server name has already been provided don't allow editing of that name
	  if { ! [string match $tServerName "NAME_TO_ADD" ] } \
	    {
	    .tlLocalEdit.fmMain.fmTop.fmName.enServerName config -state disabled -background grey
	    }
          label .tlLocalEdit.fmMain.fmTop.fmName.lbLocName -text "/"
	  entry .tlLocalEdit.fmMain.fmTop.fmName.enLocName -textvariable LocalName -background white
      
          bind .tlLocalEdit.fmMain.fmTop.fmName.enLocName <KeyPress> { set LocalName [string toupper $LocalName ] }
          bindtags .tlLocalEdit.fmMain.fmTop.fmName.enLocName [list Entry . .tlLocalEdit.fmMain.fmTop.fmName.enLocName all]
	  pack .tlLocalEdit.fmMain.fmTop.fmName.lbName -side left 
	  pack .tlLocalEdit.fmMain.fmTop.fmName.enServerName -side left 
	  pack .tlLocalEdit.fmMain.fmTop.fmName.lbLocName -side left 
	  pack .tlLocalEdit.fmMain.fmTop.fmName.enLocName -side left 
	pack .tlLocalEdit.fmMain.fmTop.fmName -side top -padx 5 -pady 5 -anchor w
        frame .tlLocalEdit.fmMain.fmTop.fmComment
          label .tlLocalEdit.fmMain.fmTop.fmComment.lbComment -text "Description:"
	  entry .tlLocalEdit.fmMain.fmTop.fmComment.enComment -textvariable LocalComment -width 50 -background white
	  pack .tlLocalEdit.fmMain.fmTop.fmComment.lbComment -side left 
	  pack .tlLocalEdit.fmMain.fmTop.fmComment.enComment -side left 
	pack .tlLocalEdit.fmMain.fmTop.fmComment -side top -padx 5 -pady 5 -anchor w
        #--- TYPES are "GRID/IMAGE/NAV/POINT/TEXT"
        frame .tlLocalEdit.fmMain.fmTop.fmTypeFormat
  	  frame .tlLocalEdit.fmMain.fmTop.fmTypeFormat.fmType
   	    set f .tlLocalEdit.fmMain.fmTop.fmTypeFormat.fmType
            menubutton $f.mb -text "Type" -menu $f.mb.menu \
                                   -relief raised -borderwidth 2 -indicatoron 1 -width 10
            pack  $f.mb -side left
	    entry $f.enEcho -textvariable LocalType -state disabled -width 6 -background white
	    pack $f.enEcho -side left -padx 10
            set m [menu $f.mb.menu -tearoff 0]
            foreach i { IMAGE GRID POINT TEXT NAV } \
	      {
              $m add command -label $i -command "set LocalType $i; FillFormatMenu"
	      }
	  pack .tlLocalEdit.fmMain.fmTop.fmTypeFormat.fmType -side left -padx 5 -pady 5 -anchor w
  	  frame .tlLocalEdit.fmMain.fmTop.fmTypeFormat.fmFormat
   	    set f .tlLocalEdit.fmMain.fmTop.fmTypeFormat.fmFormat
            menubutton $f.mb -text "Format" -menu $f.mb.menu \
                                   -relief raised -borderwidth 2 -indicatoron 1 -width 10
            pack  $f.mb -side left
	    entry $f.enEcho -textvariable LocalFormat -state disabled -width 6 -background white
	    pack $f.enEcho -side left -padx 10
            set m [menu $f.mb.menu -tearoff 0]
            foreach i $FormatList \
	      {
              $m add command -label $i -command "set LocalFormat $i"
	      }
   	  pack .tlLocalEdit.fmMain.fmTop.fmTypeFormat.fmFormat -side right -padx 5 -pady 5 -anchor w
	pack .tlLocalEdit.fmMain.fmTop.fmTypeFormat -side top -padx 5 -pady 5 -anchor w 
	frame .tlLocalEdit.fmMain.fmTop.fmBegin 
  	  label .tlLocalEdit.fmMain.fmTop.fmBegin.lbBegin -text "Start Position:" -width 15
	  entry .tlLocalEdit.fmMain.fmTop.fmBegin.enBegin -textvariable LocalBegin -width 8 -background white
          pack .tlLocalEdit.fmMain.fmTop.fmBegin.lbBegin -side left
	  pack .tlLocalEdit.fmMain.fmTop.fmBegin.enBegin -side left -padx 5
	pack .tlLocalEdit.fmMain.fmTop.fmBegin -side top -padx 5 -pady 5 -anchor w
	frame .tlLocalEdit.fmMain.fmTop.fmEnd 
  	  label .tlLocalEdit.fmMain.fmTop.fmEnd.lbEnd -text "End Position:" -width 15
	  entry .tlLocalEdit.fmMain.fmTop.fmEnd.enEnd -textvariable LocalEnd -width 8 -background white
          pack .tlLocalEdit.fmMain.fmTop.fmEnd.lbEnd -side left
	  pack .tlLocalEdit.fmMain.fmTop.fmEnd.enEnd -side left -padx 5
	pack .tlLocalEdit.fmMain.fmTop.fmEnd -side top -padx 5 -pady 5 -anchor w
      pack .tlLocalEdit.fmMain.fmTop -side top -padx 5 -pady 5 -fill x -expand true
      frame .tlLocalEdit.fmMain.fmBottom 
        button .tlLocalEdit.fmMain.fmBottom.btOK -text "OK" -width 8 \
	  -command { set KeepFlag 1; destroy .tlLocalEdit }
	button .tlLocalEdit.fmMain.fmBottom.btCancel -text "Cancel" -width 8 \
	  -command "set KeepFlag 0; destroy .tlLocalEdit"
	pack .tlLocalEdit.fmMain.fmBottom.btOK -side left -padx 10 -pady 5
 	pack .tlLocalEdit.fmMain.fmBottom.btCancel -side right -padx 10 -pady 5
      pack .tlLocalEdit.fmMain.fmBottom -side top -pady 20 -padx 5 -fill both -expand true
    pack .tlLocalEdit.fmMain -fill both -expand true
  
  
  bind .tlLocalEdit.fmMain.fmTop.fmEnd.enEnd <KeyRelease> { if { [string match $LocalType "IMAGE"] && $LocalEnd > 9999 } \
                                                              { set LocalEnd 9999 } }
  bind .tlLocalEdit.fmMain.fmTop.fmComment.enComment <KeyRelease>  { if { [string length $LocalComment] > 80 } { set LocalComment [string range $LocalComment 0 79] }}

  
  tkwait window .tlLocalEdit
  if { $KeepFlag == 1 && ![string match $LocalName "????"] } \
    {
    #--- bounds check -- if the ServerType is IMAGE then EndPosition can't be greater than 9999
    if { [string match $LocalType "IMAGE"] && $LocalEnd > 9999 } { set LocalEnd 9999 }
    #--- if the description is more than 80 characters trim it to 80
    if { [string length $LocalComment] > 80 } { set LocalComment [string range $LocalComment 0 79] }
    #--- is the servername old -- then REMOVE the old entry
    if { !$NewServerName } \
      {
      if { ! [string match $RemoveName "????" ] } \
        {
        catch { exec dsserve.k DEL $ServerName/$RemoveName }
	}
      } \
    else \
      {
      # --- going to add the server here before continuing
      set ServerName $tServerName
      VerifyServerEdit $position 
      }
       
    #--- keep the entry -- i.e. attach and change ADDE SERVER with DSSERVE command
    set ServerList($position-$offset) [format "%s %s %s %s %s" [string range $LocalType 0 0] \
             $LocalName $LocalBegin $LocalEnd $LocalComment ]

    #--- attach the new one
    catch { exec dsserve.k ADD $ServerName/$LocalName $LocalFormat $LocalBegin $LocalEnd TYPE=$LocalType \"$LocalComment }
    #--- Rebuild this location in the list	     
    ResetServerLists $position 	     
    #--- And set the flags accordingly
    SetFlags $position 
    } \
  else \
    {
    # -- need to remove the entry -- but only from the LIST SPACE so the ResetServerLists will replace if it's valid
    unset ServerList($position-$offset) 
    #--- Rebuild this location in the list	     
    ResetServerLists $position 	     
    #--- And set the flags accordingly
    SetFlags $position 
    }  
  }
#---------------------------------------------------------------------#
# FillFormatMenu -- allows type to alter available Format types       #
#---------------------------------------------------------------------#
proc FillFormatMenu { } \
  {
  global LocalType LocalFormat
  
  switch -- [string range $LocalType 0 0 ] \
    {
    I { set FormatList "{AREA} {NCDF}"}
    G { set FormatList "{GRID} {NCDF}"}
    P { set FormatList "MD"}
    T { set FormatList "TEXT"}
    N { set FormatList "SYSN"}
    }
  set LocalFormat [lindex $FormatList 0]
    
  set f .tlLocalEdit.fmMain.fmTop.fmTypeFormat.fmFormat
  catch {destroy $f.mb.menu }
  
  set m [menu $f.mb.menu -tearoff 0]
  foreach i $FormatList \
    {
    $m add command -label $i -command "set LocalFormat $i"
    }
  }
  
#----------------------------------------------------------------------------------#
# DeleteLocalServerEntry { } -- removes hilighted LOCAL server entries             #
#----------------------------------------------------------------------------------#
proc DeleteLocalServerEntry { position } \
  {
  global ServerList
  global ServerName
  
  set L .tlServerEntryEdit.fmList.lList
  
  set MarkedToDelete [ $L curselection ] 
  foreach i $MarkedToDelete \
    {
    set LocalName [lindex $ServerList($position-$i) 1]
    catch { exec dsserve.k DEL $ServerName/$LocalName }
    unset ServerList($position-$i)
    }
  ResetServerLists $position  
  SetFlags $position
  }
#----------------------------------------------------------------------------------#
# AcceptListReset { } -- Asks the user if they really want to load the ADDE lists  #
#----------------------------------------------------------------------------------#
proc AcceptListReset { } \
  {
  global retval
  
  wm title . " "

  frame .tlAccept -relief raised -borderwidth 3

  #------------ Header -------------------------------
  frame .tlAccept.frHeader -relief raised -borderwidth 3 
    label .tlAccept.frHeader.lbSerial -text "ADDE Server List Retrieval" 
    pack .tlAccept.frHeader.lbSerial -side top -pady 5 -padx 10
    
  pack .tlAccept.frHeader -side top -fill x -padx 5 -pady 5 

  #--------------- Body ----------------------------
  frame .tlAccept.frBody -relief sunken -borderwidth 3
    label .tlAccept.frBody.lbBody -text "WARNING: This may take a while"
    message .tlAccept.frBody.msBody -text " \
         You are about to start the new McIDAS GUI    \n \
         for the first time.  As part of this process \n \
         your ADDE Server List will be analyzed.  This\n \
         may take a long time.  To speed the process  \n \
	 be sure that all of your DATALOC entries are \n \
	 correct.  To continue please press 'Continue'"\
	 -aspect 600                    
    pack .tlAccept.frBody.lbBody -side top -expand true -fill x	 
    pack .tlAccept.frBody.msBody -expand true -fill both -padx 5 -pady 5
  pack .tlAccept.frBody -side top -fill x -padx 8
  #-------------Response----------------------------
  frame .tlAccept.frResponse -relief flat
    button .tlAccept.frResponse.btAccept -text "Continue" -width 14 \
              -command "destroy .tlAccept; set retval 1"
    button .tlAccept.frResponse.btExit   -text "Skip This Step" -width 14 \
              -command "destroy .tlAccept; set retval 0"
    
    pack .tlAccept.frResponse.btAccept -side left  -padx 5 -pady 5
    pack .tlAccept.frResponse.btExit   -side right -padx 5 -pady 5
  pack .tlAccept.frResponse -side top -fill x -padx 5 -pady 5  
  pack .tlAccept -side top -fill both -expand true

  tkwait window .tlAccept
  return $retval
  }



  




