# Copyright(c) 2003, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: WeatherWatch.tcl,v 1.15 2006/04/20 19:34:56 kevinb Rel $

##############################################################################
# Main entry point
##############################################################################
proc WeatherWatch {} \
{
  global SevereWatch   SevereWarning
  global TornadoWatch  TornadoWarning
  global WinterWarning WinterWatch
  global Blizzard      SnowAdvisory
  global FloodWatch    FloodWarning
  
  global BeginTime BeginDate
  global EndTime   EndDate
  global date
  global EchoBeginDate EchoEndDate
  
  global WatchStyle WarnStyle watch_list
  global WeatherWatchInit
  
  global MapName draw_map nav_type fill_density

  if { [winfo exists .tlWeatherWatch] } \
  { 
	wm deiconify .tlWeatherWatch;
	focus .tlWeatherWatch;
	raise .tlWeatherWatch;
	return
  }

  if { ![info exists WeatherWatchInit] } \
  {
    # --- set the init status
    set WeatherWatchInit 1

    # --- Initialize watches/warnings
    set SevereWatch    1
    set SevereWarning  1
    set TornadoWatch   1
    set TornadoWarning 1
    set WinterWatch    1
    set WinterWarning  1
    set SnowAdvisory   1
    set Blizzard       1
    set FloodWatch     1
    set FloodWarning   1
  
    # --- Initialize watch/warn fill draw types
    set WatchStyle "BOX"
    set WarnStyle  "SOLID"
    set watch_list "YES"
    set draw_map   "YES"
    set nav_type   "MAP"
    set fill_density 2
  }
  
  # --- Initialize date and time stuff  
  #set BeginTime [Now]
  set BeginTime "Current Time"
  set EndTime $BeginTime
  set date [Today]
  set BeginDate $date 
  set EndDate $BeginDate
  set EchoBeginDate [getnicedate $date 0]
  set EchoEndDate $EchoBeginDate

  # Create main window    
  toplevel .tlWeatherWatch \
	-relief raised \
	-borderwidth 3
  wm title .tlWeatherWatch " Weather Watch Criteria"
  wm geometry .tlWeatherWatch +70+70
  wm resizable .tlWeatherWatch no no

  # Create notebook
  Notebook:create .tlWeatherWatch.notebook \
   	-pages { "Event Types" \
		 "Day & Time" \
		 "Map Settings" \
		 "Plotting Options" } \
   	-pad 10 \
   	-width 385 \
   	-height 300
  pack .tlWeatherWatch.notebook \
   	-fill both \
   	-expand true

  # Display and Dismiss buttons
  frame .tlWeatherWatch.fmControls
  pack .tlWeatherWatch.fmControls \
	-pady 5 \
	-padx 5 \
	-side top \
	-fill both \
	-expand true

  button .tlWeatherWatch.fmControls.btDisplay \
	-text "Display" \
	-command { DoWWDISP }
  grid .tlWeatherWatch.fmControls.btDisplay \
	-row 0 \
	-column 0

  button .tlWeatherWatch.fmControls.btDisplayExit \
	-text "Display and Dismiss" \
	-command { DoWWDISP; destroy .tlWeatherWatch }
  grid .tlWeatherWatch.fmControls.btDisplayExit \
	-row 0 \
	-column 1 \
	-padx 10

  button .tlWeatherWatch.fmControls.btExit \
	-text "Dismiss" \
	-command { destroy .tlWeatherWatch }
  grid .tlWeatherWatch.fmControls.btExit \
	-row 0 \
	-column 2

  # Instantiate notebook pages
  CREATE_EVENT_TYPES_PAGE
  CREATE_DAY_TIME_PAGE
  CREATE_MAP_PAGE
  CREATE_PLOTTING_OPTIONS_PAGE
  
  # --- Initialize the status of the map controls 
  MapToggle
  
  # --- Initialize the status of the fill density controls
  FillDensityToggle
}



#######################################################
proc CREATE_EVENT_TYPES_PAGE {} \
{
  global SevereWatch
  global SevereWarning
  global TornadoWatch
  global TornadoWarning
  global WinterWarning
  global WinterWatch
  global Blizzard
  global SnowAdvisory
  global FloodWatch
  global FloodWarning
  global watch_list
  global LMActiveFrame

  set eventTypesPage [Notebook:frame .tlWeatherWatch.notebook "Event Types"]

  labelframe $eventTypesPage.eventsFrame \
   	-relief groove \
	-borderwidth 2 \
	-text "Event Types to be Displayed"
  pack $eventTypesPage.eventsFrame \
	-side top \
	-padx 10 \
	-pady 10 \
	-fill x \
	-ipady 20

  # --- Headers

  label $eventTypesPage.eventsFrame.watchHeader \
	-text "Watches"
  grid $eventTypesPage.eventsFrame.watchHeader \
	-row 1 \
	-column 1 \
	-padx 10

  label $eventTypesPage.eventsFrame.warnHeader \
	-text "Warnings"
  grid $eventTypesPage.eventsFrame.warnHeader \
	-row 1 \
	-column 2

  # --- Severe Storm

  label $eventTypesPage.eventsFrame.sevLabel \
	-text "Severe Storm"
  grid $eventTypesPage.eventsFrame.sevLabel \
	-row 3 \
	-column 0 \
	-sticky w

  checkbutton $eventTypesPage.eventsFrame.sevWatch \
	-variable SevereWatch \
	-highlightthickness 0
  grid $eventTypesPage.eventsFrame.sevWatch \
	-row 3 \
	-column 1

  checkbutton $eventTypesPage.eventsFrame.sevWarn \
	-variable SevereWarning \
	-highlightthickness 0
  grid $eventTypesPage.eventsFrame.sevWarn \
	-row 3 \
	-column 2

  # --- Winter Storm

  label $eventTypesPage.eventsFrame.winLabel \
	-text "Winter Storm"
  grid $eventTypesPage.eventsFrame.winLabel \
	-row 4 \
	-column 0 \
	-sticky w

  checkbutton $eventTypesPage.eventsFrame.winWatch \
	-variable WinterWatch \
 	-highlightthickness 0
  grid $eventTypesPage.eventsFrame.winWatch \
	-row 4 \
	-column 1

  checkbutton $eventTypesPage.eventsFrame.winWarn \
	-variable WinterWarning \
	-highlightthickness 0
  grid $eventTypesPage.eventsFrame.winWarn \
	-row 4 \
	-column 2

  # --- Tornado

  label $eventTypesPage.eventsFrame.torLabel \
	-text "Tornado"
  grid $eventTypesPage.eventsFrame.torLabel \
	-row 5 \
	-column 0 \
	-sticky w

  checkbutton $eventTypesPage.eventsFrame.torWatch \
	-variable TornadoWatch \
	-highlightthickness 0
  grid $eventTypesPage.eventsFrame.torWatch \
	-row 5 \
	-column 1

  checkbutton $eventTypesPage.eventsFrame.torWarn \
	-variable TornadoWarning \
	-highlightthickness 0
  grid $eventTypesPage.eventsFrame.torWarn \
	-row 5 \
	-column 2

  # --- Flood

  label $eventTypesPage.eventsFrame.floodLabel \
	-text "Flood"
  grid $eventTypesPage.eventsFrame.floodLabel \
	-row 6 \
	-column 0 \
	-sticky w

  checkbutton $eventTypesPage.eventsFrame.floodWatch \
	-variable FloodWatch \
 	-highlightthickness 0
  grid $eventTypesPage.eventsFrame.floodWatch \
	-row 6 \
	-column 1

  checkbutton $eventTypesPage.eventsFrame.floodWarn \
	-variable FloodWarning \
	-highlightthickness 0
  grid $eventTypesPage.eventsFrame.floodWarn \
	-row 6 \
	-column 2

  # --- Snow Advisory

  label $eventTypesPage.eventsFrame.snowLabel \
	-text "Snow Adv."
  grid $eventTypesPage.eventsFrame.snowLabel \
	-row 7 \
	-column 0 \
	-sticky w

  checkbutton $eventTypesPage.eventsFrame.snowAdv \
	-variable SnowAdvisory \
 	-highlightthickness 0
  grid $eventTypesPage.eventsFrame.snowAdv \
	-row 7 \
	-column 1

  label $eventTypesPage.eventsFrame.snowAdvWarn \
	-text "n/a"
  grid $eventTypesPage.eventsFrame.snowAdvWarn \
	-row 7 \
	-column 2

  # --- Blizzard

  label $eventTypesPage.eventsFrame.blizLabel \
	-text "Blizzard"
  grid $eventTypesPage.eventsFrame.blizLabel \
	-row 8 \
	-column 0 \
	-sticky w

  label $eventTypesPage.eventsFrame.blizWatch \
	-text "n/a"
  grid $eventTypesPage.eventsFrame.blizWatch \
	-row 8 \
	-column 1

  checkbutton $eventTypesPage.eventsFrame.blizWarn \
	-variable Blizzard \
	-highlightthickness 0
  grid $eventTypesPage.eventsFrame.blizWarn \
	-row 8 \
	-column 2

  label $eventTypesPage.eventsFrame.spacer
  grid $eventTypesPage.eventsFrame.spacer \
	-row 9

  # -- Frame selection menu button ----------------------

  frame $eventTypesPage.loopFrame
  pack $eventTypesPage.loopFrame \
	-side top \
	-pady 10

  label $eventTypesPage.loopFrame.label \
	-relief sunken \
	-borderwidth 2 \
	-width 4 \
	-textvariable LMActiveFrame
  pack $eventTypesPage.loopFrame.label \
	-side right

  frame $eventTypesPage.loopFrame.loopButtonFrame
  pack $eventTypesPage.loopFrame.loopButtonFrame

  set frameButton [LMCreateLoopButton \
			$eventTypesPage.loopFrame.loopButtonFrame \
			"Show Frame" \
			NO]
  pack $frameButton \
	-side left
}



#######################################################
proc CREATE_DAY_TIME_PAGE {} \
{
  global BeginTime
  global BeginDate
  global EndTime
  global EndDate
  global EchoBeginDate
  global EchoEndDate
  global date
  global watch_list

  set dayTimePage [Notebook:frame .tlWeatherWatch.notebook "Day & Time"]

  # --- Main frame for date/time controls ------------------------

  labelframe $dayTimePage.dateTimeFrame \
	-relief groove \
	-borderwidth 2 \
	-text "Date and Time Range"
  pack $dayTimePage.dateTimeFrame\
	-side top \
	-padx 10 \
	-pady 10 \
	-fill x \
	-ipady 20

  # --- Beginning Day and Time ----------------

  frame $dayTimePage.dateTimeFrame.fmBegin
  grid $dayTimePage.dateTimeFrame.fmBegin \
	-row 0 \
	-column 0 \
	-padx 5 \
	-pady 10

  label $dayTimePage.dateTimeFrame.fmBegin.lbBegin \
	-text "Beginning:"
  pack $dayTimePage.dateTimeFrame.fmBegin.lbBegin \
	-side top \
	-anchor w

  # --- Date Frame

  frame $dayTimePage.dateTimeFrame.fmBegin.fmDate
  pack $dayTimePage.dateTimeFrame.fmBegin.fmDate \
	-side top

  set f $dayTimePage.dateTimeFrame.fmBegin.fmDate

  menubutton $f.mb \
	-text "Date" \
	-menu $f.mb.menu \
        -relief raised \
	-borderwidth 2 \
	-indicatoron 1 \
	-width 6
  pack  $f.mb \
	-side left

  label $f.entryDate \
	-textvar EchoBeginDate \
	-width 13 \
	-relief sunken \
	-borderwidth 2 \
	-anchor w
  pack  $f.entryDate \
	-side left \
	-fill x \
	-expand true

  menu $f.mb.menu -tearoff 0
  
  for { set i -6} { $i < 1 } { incr i 1 } \
  {
	$f.mb.menu add command \
		-label [getnicedate $date $i] \
		-command "set BeginDate [expr $date+$i ]
	                  set EchoBeginDate {[getnicedate $date $i]}"
  }

  # --- Time Frame

  frame $dayTimePage.dateTimeFrame.fmBegin.fmTime
  pack $dayTimePage.dateTimeFrame.fmBegin.fmTime \
	-side top

  set f $dayTimePage.dateTimeFrame.fmBegin.fmTime

  menubutton $f.mb \
	-text "Time" \
	-menu $f.mb.menu \
	-relief raised \
	-borderwidth 2 \
	-indicatoron 1 \
	-width 6
  pack  $f.mb \
	-side left

  label $f.entryTime \
        -textvar BeginTime \
	-width 13 \
	-relief sunken \
	-borderwidth 2 \
	-anchor w
  pack  $f.entryTime \
	-side left \
	-fill x \
	-expand true

  menu $f.mb.menu -tearoff 0

  set columnBreak "NO"
  $f.mb.menu add radio \
	-label "Current" \
        -command "set BeginTime \"Current Time\"" \
        -variable BeginTime \
        -columnbreak $columnBreak
        #-command "set BeginTime [Now]" \

  for {set i 0} {$i < 24} {incr i} \
  {
	if { $i == 12 } { set columnBreak "YES" }

	$f.mb.menu add radio \
	   -label $i \
	   -variable BeginTime \
	   -columnbreak $columnBreak

	set columnBreak "NO"
  }


  # --- Ending Day and Time ----------------

  frame $dayTimePage.dateTimeFrame.fmEnd
  grid $dayTimePage.dateTimeFrame.fmEnd \
	-row 1 \
	-column 0 \
	-padx 5 \
	-pady 20

  label $dayTimePage.dateTimeFrame.fmEnd.lbEnd \
	-text "Ending:"
  pack $dayTimePage.dateTimeFrame.fmEnd.lbEnd \
	-side top \
	-anchor w

  # --- Date Frame

  frame $dayTimePage.dateTimeFrame.fmEnd.fmDate
  pack $dayTimePage.dateTimeFrame.fmEnd.fmDate \
	-side top

  set f $dayTimePage.dateTimeFrame.fmEnd.fmDate

  menubutton $f.mb \
	-text "Date" \
	-menu $f.mb.menu \
        -relief raised \
	-borderwidth 2 \
	-indicatoron 1 \
	-width 6
  pack  $f.mb \
	-side left

  label $f.entryDate \
	-textvar EchoEndDate \
	-width 13 \
	-relief sunken \
	-borderwidth 2 \
	-anchor w
  pack  $f.entryDate \
	-side left \
	-fill x \
	-expand true

  menu $f.mb.menu -tearoff 0
  
  for { set i -6} { $i < 1 } { incr i 1 } \
  {
	$f.mb.menu add command \
		-label [getnicedate $date $i] \
		-command "set EndDate [expr $date+$i ]
	                  set EchoEndDate {[getnicedate $date $i]}"
  }

  # --- Time Frame

  frame $dayTimePage.dateTimeFrame.fmEnd.fmTime
  pack $dayTimePage.dateTimeFrame.fmEnd.fmTime \
	-side top

  set f $dayTimePage.dateTimeFrame.fmEnd.fmTime

  menubutton $f.mb \
	-text "Time" \
	-menu $f.mb.menu \
	-relief raised \
	-borderwidth 2 \
	-indicatoron 1 \
	-width 6
  pack  $f.mb \
	-side left

  label $f.entryTime \
        -textvar EndTime \
	-width 13 \
	-relief sunken \
	-borderwidth 2 \
	-anchor w
  pack  $f.entryTime \
	-side left \
	-fill x \
	-expand true

  menu $f.mb.menu -tearoff 0

  set columnBreak "NO"
  $f.mb.menu add radio \
	-label "Current" \
        -command "set EndTime \"Current Time\"" \
        -variable EndTime \
        -columnbreak $columnBreak
        #-command "set EndTime [Now]" \

  for {set i 0} {$i < 24} {incr i} \
  {
	if { $i == 12 } { set columnBreak "YES" }

	$f.mb.menu add radio \
	   -label $i \
	   -variable EndTime \
	   -columnbreak $columnBreak

	set columnBreak "NO"
  }
}


#######################################################
proc CREATE_MAP_PAGE {} \
{
  global mapPage
  global MapName
  global MapColor
  global MapColorName
  global num_colors
  global color_table
  global draw_map
  global nav_type
  global tcl_version

  set mapPage [Notebook:frame .tlWeatherWatch.notebook \
			"Map Settings"]

  labelframe $mapPage.mapFrame \
	-relief groove \
	-borderwidth 2 \
	-text "Map Options"
  pack $mapPage.mapFrame \
	-padx 10 \
	-pady 10 \
	-fill x

  # Draw map check button ------------------------------

  checkbutton $mapPage.mapFrame.displayMap \
	-text "Draw map" \
	-highlightthickness 0 \
	-variable draw_map \
	-onvalue "YES" \
	-offvalue "NO" \
	-command MapToggle
  grid $mapPage.mapFrame.displayMap \
	-row 0 \
	-column 0 \
	-padx 10 \
	-pady 10 \
	-sticky w
	
  # --- Map name selection ------------------------------

  frame $mapPage.mapFrame.mapNameFrame
  grid $mapPage.mapFrame.mapNameFrame \
	-row 1 \
	-column 0 \
	-padx 10 \
	-pady 10 \
	-sticky ew

  menubutton $mapPage.mapFrame.mapNameFrame.menuButton \
	-text "Name" \
	-menu $mapPage.mapFrame.mapNameFrame.menuButton.menu \
	-relief raised \
	-borderwidth 2 \
	-indicatoron 1 \
	-width 10
  pack $mapPage.mapFrame.mapNameFrame.menuButton \
	-side left

  # Fill the menu for the Map Name menu button
  FillMapMenu $mapPage.mapFrame.mapNameFrame.menuButton.menu

  label $mapPage.mapFrame.mapNameFrame.label \
	-textvar MapName \
	-width 30 \
	-relief sunken \
	-borderwidth 2 \
	-anchor w
  pack $mapPage.mapFrame.mapNameFrame.label \
	-side left

  # Set the default map to 'USA'
  set MapName USA

   # --- Map color selection -----------------------------

   ReadColorTable

   frame $mapPage.mapFrame.colorFrame
   grid $mapPage.mapFrame.colorFrame \
	-row 2 \
	-column 0 \
	-padx 10 \
	-sticky ew

   menubutton $mapPage.mapFrame.colorFrame.mapColorMenuButton \
	-text "Color" \
	-width 10 \
	-relief raised \
	-borderwidth 2 \
	-indicatoron yes \
	-menu $mapPage.mapFrame.colorFrame.mapColorMenuButton.menu
   pack $mapPage.mapFrame.colorFrame.mapColorMenuButton \
	-side left

   label $mapPage.mapFrame.colorFrame.lbMapColor \
	-textvar MapColorName \
	-width 30 \
	-relief sunken \
	-borderwidth 2 \
	-anchor w
   pack $mapPage.mapFrame.colorFrame.lbMapColor \
	-side left

   # Create and fill map color menu
   menu $mapPage.mapFrame.colorFrame.mapColorMenuButton.menu \
	-tearoff no

   # Clear menu
   $mapPage.mapFrame.colorFrame.mapColorMenuButton.menu delete 0 end
 
   # Fill the map color menu
   for {set i 1} {$i < [expr {$num_colors}]} {incr i} \
   {
	$mapPage.mapFrame.colorFrame.mapColorMenuButton.menu add command \
	   -label "[lindex $color_table($i) 0] [lindex $color_table($i) 1]" \
   	   -background [format "#%02x%02x%02x" \
			[lindex $color_table($i) 4] \
			[lindex $color_table($i) 3] \
			[lindex $color_table($i) 2]] \
	   -command "
				set MapColor [lindex $color_table($i) 0]
				set MapColorName [ColorLevelToName [lindex $color_table($i) 0]]
			"
   }

  # Get the default map color from the MAP.* context files
  set MapColor [ReadContextFileVariable MCOLOR]
  if {[string match $MapColor "NONE"]} {set MapColor 1}
  set MapColorName [ColorLevelToName $MapColor]

  # Navigation selection ------------------------------

  labelframe $mapPage.mapFrame.navFrame \
	-text " Navigation " \
	-relief groove \
	-borderwidth 2
  grid $mapPage.mapFrame.navFrame \
	-row 3 \
	-column 0 \
	-padx 10 \
	-pady 15 \
	-sticky ew

  radiobutton $mapPage.mapFrame.navFrame.mapNav \
	-text "Use MAP to create navigation" \
	-highlightthickness 0 \
	-variable nav_type \
	-value "MAP"
  pack $mapPage.mapFrame.navFrame.mapNav \
	-side top \
	-anchor w \
	-padx 30 \
	-pady 5

  radiobutton $mapPage.mapFrame.navFrame.frameNav \
	-text "Use existing frame navigation" \
	-highlightthickness 0 \
	-variable nav_type \
	-value "FRAME"
  pack $mapPage.mapFrame.navFrame.frameNav \
	-side top \
	-anchor w \
	-padx 30 \
	-pady 5
}



#######################################################
proc CREATE_PLOTTING_OPTIONS_PAGE {} \
{
  global optionsPage
  global fill_density
  global WatchStyle
  global WarnStyle
  global WeatherWatchInit
  global watch_list
  global smallerfont

  set optionsPage [Notebook:frame .tlWeatherWatch.notebook \
			"Plotting Options"]

  # --- Plot Method for Tornado/Severe Thunderstorm Watches

  labelframe $optionsPage.fmWatchBox \
	-relief groove \
	-borderwidth 2 \
	-text "Plotting Method for Watches"
  pack $optionsPage.fmWatchBox \
	-side top \
	-padx 10 \
	-pady 10 \
	-fill x \
	-ipady 10
	
  message $optionsPage.fmWatchBox.info \
  	-text "This only applies to tornado and severe thunderstorm watches." \
	-width 250 \
	-font $smallerfont
  grid $optionsPage.fmWatchBox.info \
  	-row 0 \
	-column 0 \
	-columnspan 2

  radiobutton $optionsPage.fmWatchBox.rbBox \
	-variable WatchStyle \
	-text "Watch Boxes" \
	-value "BOX" \
	-highlightthickness 0
  grid $optionsPage.fmWatchBox.rbBox \
	-row 1 \
	-column 0 \
	-sticky w \
	-padx 5

  radiobutton $optionsPage.fmWatchBox.rbCnty \
	-variable WatchStyle \
	-text "County Outlines" \
	-value "COUNTY" \
	-highlightthickness 0
  grid $optionsPage.fmWatchBox.rbCnty \
	-row 1 \
	-column 1 \
	-padx 5 \
	-pady 5

  # --- Warning Fill Type ------------------------

  labelframe $optionsPage.fmFill \
	-relief groove \
	-borderwidth 2 \
	-text "Fill Type for Warnings"
  pack $optionsPage.fmFill \
	-side top \
	-padx 10 \
	-pady 10 \
	-fill x \
	-ipady 10

  radiobutton $optionsPage.fmFill.rbSolid \
	-variable WarnStyle \
	-text "Solid" \
	-value "SOLID" \
	-highlightthickness 0 \
	-command { FillDensityToggle }
  grid $optionsPage.fmFill.rbSolid \
	-row 0 \
	-column 0 \
	-sticky nw \
	-padx 20

  radiobutton $optionsPage.fmFill.rbNone \
	-variable WarnStyle \
	-text "No Fill" \
	-value "NONE" \
	-highlightthickness 0 \
	-command { FillDensityToggle }
  grid $optionsPage.fmFill.rbNone \
	-row 1 \
	-column 0 \
	-sticky nw \
	-padx 20

  radiobutton $optionsPage.fmFill.rbTrans \
	-variable WarnStyle \
	-text "Transparent" \
	-value "TRAN" \
	-highlightthickness 0 \
	-command { FillDensityToggle }

  # --- Density controls if 'Transparent' option is selected

  labelframe $optionsPage.fmFill.fmDensity \
  	-labelwidget $optionsPage.fmFill.rbTrans
  grid $optionsPage.fmFill.fmDensity \
	-row 0 \
	-column 1 \
	-rowspan 2 \
	-sticky sw \
	-padx 20 \
	-ipadx 5 \
	-ipady 5

  label $optionsPage.fmFill.fmDensity.label \
	-text "Fill Density:" \
	-font $smallerfont \
	-foreground grey
  grid $optionsPage.fmFill.fmDensity.label \
	-row 0 \
	-column 0

  spinbox $optionsPage.fmFill.fmDensity.densityControl \
	-from 2 \
	-to 10 \
	-textvariable fill_density \
	-width 2 \
	-highlightthickness 0 \
	-font $smallerfont \
	-wrap yes
  grid $optionsPage.fmFill.fmDensity.densityControl \
	-row 0 \
	-column 1

  checkbutton $optionsPage.watchList \
	-text "Display List of Watches in Upper Left Corner" \
	-highlightthickness 0 \
	-variable watch_list \
	-onvalue "YES" \
	-offvalue "NO"
  pack $optionsPage.watchList \
  	-side top \
	-padx 5 \
	-pady 10
}

#--------------------------------------------------------------#
# proc FillDensityToggle -- toggles on/off the fill density    #
#                           controls dependant on the status   #
#                           of the Fill Type radio buttons.    #
#--------------------------------------------------------------#
proc FillDensityToggle {} \
{
   global WarnStyle
   global optionsPage
   
   if { [string match $WarnStyle "TRAN"] } \
   {
	$optionsPage.fmFill.fmDensity.label configure \
	   -foreground black
   	$optionsPage.fmFill.fmDensity.densityControl configure \
	   -state readonly \
	   -foreground black
   } \
   else \
   {
	$optionsPage.fmFill.fmDensity.label configure \
	   -foreground grey
	$optionsPage.fmFill.fmDensity.densityControl configure \
	   -foreground grey \
	   -state disabled
   }
}


#--------------------------------------------------------------#
# proc DoWWDISP -- executes the WWDISP command with all of the #
#                  entered parameters                          #
#--------------------------------------------------------------#
proc DoWWDISP { } \
{
  global SevereWatch		
  global SevereWarning
  global TornadoWatch 	
  global TornadoWarning
  global WinterWarning 	
  global WinterWatch
  global Blizzard			
  global SnowAdvisory
  global FloodWatch		
  global FloodWarning
  global BeginTime 
  global EndTime
  global BeginDate 
  global EndDate
  global WatchStyle 
  global WarnStyle
  global MapName 
  global MapColor
  global draw_map
  global nav_type
  global fill_density
  global watch_list


  # --- If using MAP to create nav, erase frame first

  if { [string match $nav_type "MAP"] } { RunCommand "ERASE" }

  # --- Set the correct map options

  if { [string match $MapName "Default Map"] } \
  { 
	set map_command "MAP X $MapColor"
  } \
  elseif { [string match $MapName "VH - USA (high res.)"] } \
  {
	set map_command "MAP VH $MapColor"
  } \
  else \
  {
	set map_command "MAP $MapName $MapColor"
  }

  if { [string match $nav_type "FRAME"] } \
  {
	set map_command "$map_command PRO=SAT"
  } \
  else \
  {
	set map_command "$map_command PRO=MERC"
  }

  # --- build the TYPE Keyword here

  set Type "TYPE="

  # --- Day range  

  set bday [expr $BeginDate % 1000]
  set eday [expr $EndDate % 1000]
  set Day "DAY=$bday $eday"

  # --- Time Range

  if { $eday < $bday } { set bday $eday }

  if { $BeginTime == "Current Time" } \
  {
    set UseBeginTime [Now]
  } \
  else \
  {
    set UseBeginTime $BeginTime
  }

  if { $EndTime == "Current Time" } \
  {
    set UseEndTime [Now]
  } \
  else \
  {
    set UseEndTime $EndTime
  }

  if { $eday == $bday && $UseEndTime < $UseBeginTime } \
  {
	set UseEndTime $UseBeginTime
  }
  set Time "TIME=$UseBeginTime $UseEndTime"

  # --- Event types to be plotted

  if { $SevereWatch    } { append Type " SEVWATCH"   }
  if { $SevereWarning  } { append Type " SEVWARN"    }
  if { $TornadoWatch   } { append Type " TORWATCH"   }
  if { $TornadoWarning } { append Type " TORWARN"    }
  if { $WinterWatch    } { append Type " WINWATCH"   }
  if { $WinterWarning  } { append Type " WINWARN"    }
  if { $SnowAdvisory   } { append Type " SNOWADV"    }
  if { $Blizzard       } { append Type " BLIZZARD"   }
  if { $FloodWatch     } { append Type " FLOODWATCH" }
  if { $FloodWarning   } { append Type " FLOODWARN"  }

  # --- Build WWDISP command

  set wwdisp_command "WWDISP $Type $Day $Time \
	  		  PLOT=$WatchStyle \
			  FILL=$WarnStyle $fill_density \
			  WLIST=$watch_list"

  # --- Run command or warn if no event types were selected

  if { [llength $Type] > 1 } \
  {
	if { [string match $draw_map "YES"] } \
	{
	   RunCommand $map_command
	}

  	RunCommand $wwdisp_command
  } \
  else \
  {  
	tk_messageBox \
	   -message "At least one event type must be selected." \
	   -parent .tlWeatherWatch \
	   -icon error \
	   -title " Error"
  }
}



#--------------------------------------------------------------#
# proc MapToggle -- Toggle the state of the map controls       #
#                   based on the condition of the map check    #
#                   button.                                    #
#--------------------------------------------------------------#
proc MapToggle { } \
{

  global draw_map
  global mapPage
  global tcl_version

  if { [string match $draw_map "YES"] } \
  {
	$mapPage.mapFrame.mapNameFrame.menuButton configure \
	   -state normal
	$mapPage.mapFrame.mapNameFrame.label  configure \
	   -foreground black

	$mapPage.mapFrame.colorFrame.mapColorMenuButton configure \
	   -state normal
	$mapPage.mapFrame.colorFrame.lbMapColor configure \
	   -foreground black

	if { [expr $tcl_version >= 8.4] } \
	{
	    $mapPage.mapFrame.navFrame configure \
	   	-foreground black
	}

	$mapPage.mapFrame.navFrame.frameNav configure \
	   -state normal
	$mapPage.mapFrame.navFrame.mapNav configure \
	   -state normal
  } \
  else \
  {
	$mapPage.mapFrame.mapNameFrame.menuButton configure \
	   -state disabled
	$mapPage.mapFrame.mapNameFrame.label configure \
	   -foreground grey

	$mapPage.mapFrame.colorFrame.mapColorMenuButton configure \
	   -state disabled
	$mapPage.mapFrame.colorFrame.lbMapColor configure \
	   -foreground grey

	if { [expr $tcl_version >= 8.4] } \
	{
	    $mapPage.mapFrame.navFrame configure \
	   	-foreground grey
	}
	$mapPage.mapFrame.navFrame.frameNav select
	$mapPage.mapFrame.navFrame.frameNav configure \
	   -state disabled
	$mapPage.mapFrame.navFrame.mapNav configure \
	   -state disabled
  }
}
