# Copyright(c) 2003, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: formatLatLon.tcl,v 1.2 2003/10/23 22:38:52 tsmith Exp $

# remove all zeros from a number,
# but leave a zero if it's the only digit
#
proc deZeroLatLon {val} \
{
  set val [string trimleft $val 0]

  if { [string length $val] == 0 } \
  {
    set val 0
  }

  return $val
}

# pad left side of $str with $char until it is $max characters long
#
proc leftFill {str char max} \
{
  for {set i [string length $str]} {$i < $max} {incr i} \
  {
    set str "$char$str"
  }

  return $str
}

# take an integer value of the form DDDMMSS
# and convert it to either "DDD:MM:SS" or "DDD.dddd"
#
proc formatLatLon { lat_or_lon value {fmtType -1}} \
{
  # use UC value if format type was unspecified
  #
  if {$fmtType == -1} { set fmtType [UCpeek [ucLocation latlon_format]] }

  # if format type is unrecognized, default to displaying in DDDMMSS
  #
  if {$fmtType != 1} { set fmtType 0 }

  # make sure the value is an integer
  #
  if [catch {expr $value + 0} msg] { return "" }

  # take absolute value of lat/lon and append N/S/E/W direction label
  #
  if { $value >= 0 } \
  {
    if { [string match $lat_or_lon "LAT"] } \
    {
       set dir_label "N"
    } \
    else \
    {
       set dir_label "W"
    }
  } \
  else \
  {
    set value [expr -$value]

    if { [string match $lat_or_lon "LAT"] } \
    {
       set dir_label "S"
    } \
    else \
    {
       set dir_label "E"
    }
  }

  # left-fill integer with zeros
  #
  set fill [expr 6 - [string length $value]]
  set zval "[string range 0000000 0 $fill]$value"

  # break into component parts
  #
  set degrees [deZeroLatLon [string range $zval 0 2]]
  set minutes [deZeroLatLon [string range $zval 3 4]]
  set seconds [deZeroLatLon [string range $zval 5 6]]

  # build formatted string
  #
  set string ""
  if {$fmtType == 0} \
  {
    set string "$degrees:[leftFill $minutes 0 2]:[leftFill $seconds 0 2] $dir_label "
  } \
  else \
  {
    set decimal [expr $degrees + (($minutes + ($seconds / 60.0)) / 60.0)]
    set string "[string range $decimal 0 8] $dir_label "
  }

  return $string
}
