#!/bin/false

# Copyright(c) 1998, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: kidprocs.tcl,v 1.8 1998/10/13 20:26:38 dglo Exp $

proc trimkids {} {
	global kidlist

	# nothing to do if no children were created
	#
	if {[info exists kidlist] == 0} {
		return
	}

	# send each child a "SIGNULL", which verifies that the
	#  child is still alive
	#
	set newkids ""
	foreach i $kidlist {
		if {[catch {exec kill -0 $i >/dev/null 2>/dev/null}] == 0} {

			# still alive; keep this kid on the list
			#
			set newkids [lappend newkids $i]
		}
	}

	# save the new list, or destroy it if we're childless
	#
	if {$newkids != ""} {
		set kidlist $newkids
	} else {
		unset kidlist
	}
}

proc killkids {} {
	global kidlist

	# nothing to do if we have no children
	#
	if {[info exists kidlist] == 0} {
		return
	}

	# it's Guyana Kool-Aid time!
	#
	foreach i $kidlist {
		catch {exec kill -TERM $i >/dev/null 2>/dev/null}
	}
}

proc execGUI {guibase {arg1 ""} {arg2 ""}} {
	global env
	global guipath
	global kidlist
	global wishpath

	# make sure the list of children is initialized
	#
	if {[info exists kidlist] == 0} {
		set kidlist {}
	}

	# make sure we know how to invoke wish
	#
	if {[info exists wishpath] == 0} {
		set wishpath "wish"
		if [catch {set wishpath [info nameofexecutable]}] {
			set wishpath "wish"
		}
	}

	# expand GUI module name to a full path
	#
	set fullpath "$guipath/$guibase.gui"

	# verify that GUI module exists
	#
	if {[file exists $fullpath] == 0} {
		puts "Attempt to exec $guibase failed -- $fullpath not found"
		return;
	}

	# make sure negative UC info doesn't hose us
	#
	if [info exists env(MCENV_NEGUC)] {
		unset env(MCENV_NEGUC)
	}

	# spawn a child process running the GUI module
	#
	if {$arg1 != ""} {
		if {$arg2 != ""} {
			set newpid [exec $wishpath -f $fullpath $arg1 $arg2 &]
		} else {
			set newpid [exec $wishpath -f $fullpath $arg1 &]
		}
	} else {
		set newpid [exec $wishpath -f $fullpath &]
	}

#puts "DGLO_DBUG: Startup of $guibase forked off $newpid"

	# add the child process to the list of children
	#
	set kidlist [concat $kidlist $newpid]

	# do some housekeeping every so often
	#
	if {[llength $kidlist] > 10} {
		trimkids
	}
}
